/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package actions;

import java.awt.event.ActionEvent;
import java.io.File;
import java.io.FileInputStream;
import java.io.FileNotFoundException;
import java.io.IOException;
import dialogs.ChoixPrototypeDansRepertoireDlg;
import objets.CMathGrafDoc;
import objets.CPrototype;
import entreesSorties.MathGraphDataInputStream;
import mathgraph32.Global;
import mathgraph32.MtgFrame;

@SuppressWarnings("serial")
public class ActionConstructionIncorporer extends MtgAction {
	public ActionConstructionIncorporer(MtgFrame frameInit) {
		super(frameInit, null, getString("chaineMenuIncorporerConstruction"), null, null, null);
	}

	@Override
	public void actionPerformed(ActionEvent e) {
		if (frame.cadreActif() == null) {
			frame.dialogueAvertissement(getString("messageOutilPasDisponible"));
			frame.activeOutilCapture();
			return;
		}
		frame.actionEnCours = this;
		
		ChoixPrototypeDansRepertoireDlg dlg = new ChoixPrototypeDansRepertoireDlg(frame);
		dlg.setLocationRelativeTo(frame);
		dlg.setVisible(true);
		if (dlg.retourOK()) {
			File file = dlg.fileChooser.getSelectedFile();
			String ch = file.getAbsolutePath();
			if (!ch.toLowerCase().endsWith(Global.GLOBAL_COMMA_MGC)) {
				ch = ch + Global.GLOBAL_COMMA_MGC;
				file = new File(ch);
			}						
			if (file != null) {
				FileInputStream fis;
				try {
					fis = new FileInputStream(file);
					MathGraphDataInputStream inps = new MathGraphDataInputStream(fis);
					CMathGrafDoc doc = frame.cadreActif().document;
					CPrototype proto = new CPrototype();
					try {
						proto.read(inps);
						inps.close();
						if (doc.existePrototype(proto.nom, null)) {
							frame.dialogueAvertissement(getString("ch147"));
						}
						else doc.tablePrototypes.add(proto);
					} catch (IOException ioerr) {
						frame.dialogueAvertissement(getString("ch35"));
					}
				} catch (FileNotFoundException e1) {
					frame.dialogueAvertissement(getString("ch35"));
				}
			}
		}
		frame.activeOutilCapture();
	}
	
  @Override
  public int idMenu() {
    return 33162;
  }
	
}
