/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package dialogs;

import java.awt.Component;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.util.Vector;

import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.SwingConstants;

import objets.CArcDeCercle;
import objets.CListeObjets;

import mathgraph32.MtgFrame;


@SuppressWarnings("serial")
public class AngleArcDlg extends MtgDlg implements ActionListener {
  private EditeurAngle editeurAngle;
  private JButton btnOK, btnCancel;
  private boolean modification;
  private CArcDeCercle arc;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
  private PaneBoutonsListeValeursEtFonctionsCombo paneListeValeurEtFonctions;

  /** Creates a new instance of AngleArcDlg */
  public AngleArcDlg(MtgFrame frameInit, CArcDeCercle arcInit, boolean modificationInit) {
    super(getString("AngleArcDlg1"), true, frameInit);
    modification = modificationInit;
    setResizable(false);
    arc = arcInit;
    JLabel label = new JLabel(getString("angle"), SwingConstants.LEFT);
    editeurAngle = new EditeurAngle(frame, arc.angleAuCentre, arc, frame.cadreActif().listePr.uniteAngle);
    if (modification) editeurAngle.setSelectedItem(arc.angleAuCentre.chaineInfo());
    else editeurAngle.setSelectedIndex(-1);
    btnCancel = new JButton(getString("btnCancel"));
    btnOK = new JButton(getString("btnOK"));
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    if (modification) indmax = liste.indexOf(arc) - 1;
    else indmax = liste.size() - 1;
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsCombo(frame, editeurAngle, true, indmax, false);
    
    JPanel pane = new JPanel();
    pane.setLayout(new GridBagLayout());
    JPanel paneBas = new JPanel(); // Pour contenir les boutons OK et Cancel
    paneBas.setLayout(new GridBagLayout());
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    
    setConstraints(constraints, 0, 0, 1, 1, 40, 40, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label, constraints);
    
    setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    pane.add(editeurAngle, constraints);
    constraints.insets.bottom = 5;
    
    setConstraints(constraints, 0, 1, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.EAST);
    pane.add(paneListeValeurEtFonctions, constraints);
    
    setConstraints(constraints, 0, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneBas.add(btnOK, constraints);

    setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneBas.add(btnCancel, constraints);
    
    setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
    pane.add(paneBas, constraints);
    
    setContentPane(pane);
    btnOK.addActionListener(this);
    btnCancel.addActionListener(this);
    getRootPane().setDefaultButton(btnOK);
    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(3);
    order.add(editeurAngle.getEditor().getEditorComponent());
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    pack();
  }  
  public void actionPerformed(ActionEvent evt) {
    if (evt.getSource()== btnOK) {
      // Si la rponse est valide, on ferme la boite de dialogue sans la dtruire.
      // VrifieValidite se charge de changer l'angle de la rotation si la rponse est
      // correcte.
      if (editeurAngle.verifieValidite()) {
        
        setRetourOK(true);
        dispose();
      }
    }
    else {
      if (evt.getSource() == btnCancel) {
        dispose();
      }
    }
    // La variable retourOK a t initialie  false donc tous les autres cas n'on pas  tre traits
  }  
}
