/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.beans.PropertyVetoException;
import java.io.IOException;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JRadioButton;
import objets.CImplementationProto;
import objets.CListeObjets;
import objets.CMathGrafDoc;
import objets.CPrototype;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import mathgraph32.MtgMenuFileItem;

@SuppressWarnings("serial")
public class FigureBaseDlg extends MtgDlg {
  private javax.swing.JButton btnCancel;
  private javax.swing.JButton btnOK;
  private javax.swing.JPanel paneLeft;
  private javax.swing.JPanel paneRight;
  private javax.swing.JPanel panePrincipal;  
  private javax.swing.ButtonGroup btnGroup;
  private JRadioButton radiobtnTriangleQuelconque;
  private JRadioButton radiobtnTriangleRectangle;
  private JRadioButton radiobtnTriangleIsocele;
  private JRadioButton radiobtnTriangleRectangleIsocele;
  private JRadioButton radiobtnTriangleEquilateral;
  private JRadioButton radiobtnCarre;
  private JRadioButton radiobtnRectangle;
  private JRadioButton radiobtnParallelogramme;
  private JRadioButton radiobtnLosange;
  private JRadioButton radiobtnTrapeze;
  private JRadioButton radiobtnTrapezeRectangle;
  private JRadioButton radiobtnTrapezeIsocele;
  private JRadioButton radiobtnHexagoneRegulier;  
  private JRadioButton radiobtnPentagoneRegulier;  
  
  
 
	
  public FigureBaseDlg(MtgFrame frameInit) {
    super (getString("chaineMenuFigureBase"), true, frameInit);
    initComponents();
    getRootPane().setDefaultButton(btnOK);
  }
  
  private void initComponents()  {
    btnGroup = new javax.swing.ButtonGroup();
    radiobtnTriangleQuelconque = new javax.swing.JRadioButton(getString("triangleQuelconque"));
    radiobtnTriangleRectangle = new javax.swing.JRadioButton(getString("triangleRectangle"));
    radiobtnTriangleIsocele = new javax.swing.JRadioButton(getString("triangleIsocele"));
    radiobtnTriangleRectangleIsocele = new javax.swing.JRadioButton(getString("triangleRectangleIsocele"));
    radiobtnTriangleEquilateral = new javax.swing.JRadioButton(getString("triangleEquilateral"));
    radiobtnCarre = new javax.swing.JRadioButton(getString("carre"));
    radiobtnRectangle = new javax.swing.JRadioButton(getString("rectangle"));
    radiobtnParallelogramme = new javax.swing.JRadioButton(getString("parallelogramme"));
    radiobtnLosange = new javax.swing.JRadioButton(getString("losange"));
    radiobtnTrapeze = new javax.swing.JRadioButton(getString("trapeze"));
    radiobtnTrapezeRectangle = new javax.swing.JRadioButton(getString("trapezeRectangle"));
    radiobtnTrapezeIsocele = new javax.swing.JRadioButton(getString("trapezeIsocele"));
    radiobtnHexagoneRegulier = new javax.swing.JRadioButton(getString("hexagoneRegulier"));
    radiobtnPentagoneRegulier = new javax.swing.JRadioButton(getString("pentagoneRegulier"));
    btnGroup.add(radiobtnTriangleQuelconque);
    btnGroup.add(radiobtnTriangleRectangle);
    btnGroup.add(radiobtnTriangleIsocele);
    btnGroup.add(radiobtnTriangleRectangleIsocele);
    btnGroup.add(radiobtnTriangleEquilateral);
    btnGroup.add(radiobtnCarre);
    btnGroup.add(radiobtnRectangle);
    btnGroup.add(radiobtnParallelogramme);
    btnGroup.add(radiobtnLosange);
    btnGroup.add(radiobtnTrapeze);
    btnGroup.add(radiobtnTrapezeRectangle);
    btnGroup.add(radiobtnTrapezeIsocele);
    btnGroup.add(radiobtnHexagoneRegulier);
    btnGroup.add(radiobtnPentagoneRegulier);
    paneLeft = new javax.swing.JPanel();
    paneRight = new javax.swing.JPanel();
    panePrincipal = new javax.swing.JPanel();
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    paneLeft.setLayout(new BoxLayout(paneLeft, BoxLayout.Y_AXIS));
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneLeft.add(radiobtnTriangleQuelconque);
    paneLeft.add(radiobtnTriangleRectangle);
    paneLeft.add(radiobtnTriangleIsocele);
    paneLeft.add(radiobtnTriangleRectangleIsocele);
    paneLeft.add(radiobtnTriangleEquilateral);
    paneLeft.add(radiobtnCarre);
    paneLeft.add(radiobtnRectangle);
    paneLeft.add(radiobtnParallelogramme);
    paneLeft.add(radiobtnLosange);
    paneLeft.add(radiobtnTrapeze);
    paneLeft.add(radiobtnTrapezeRectangle);
    paneLeft.add(radiobtnTrapezeIsocele);
    paneLeft.add(radiobtnHexagoneRegulier);
    paneLeft.add(radiobtnPentagoneRegulier);
    radiobtnTriangleQuelconque.setSelected(true);
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    panePrincipal.setLayout(new FlowLayout());
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
  
  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {
    dispose ();
  }                                         

  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {
		// Modification version 3.5 : Si la prcdente fentre a t maximise, on maximise la nouvelle
		MtgInternalFrame cadreActif = frame.cadreActif();
		boolean maximise = cadreActif == null ? false : cadreActif.isMaximum();		
  	CPrototype proto = null;
  	String ch = null;
  	if (radiobtnTriangleQuelconque.isSelected()) { 
  		proto = frame.constructionTriangleQuelconque;
  		ch = getString("triangleQuelconque");
  	}
  	else
  	if (radiobtnTriangleRectangle.isSelected()) {
  		proto = frame.constructionTriangleRectangle;
  		ch = getString("triangleRectangle");
  	}
  	else
  	if (radiobtnTriangleIsocele.isSelected()) {
  		proto = frame.constructionTriangleIsocele;
  		ch = getString("triangleIsocele");
  	}
  	else
  	if (radiobtnTriangleRectangleIsocele.isSelected()) {
  		proto = frame.constructionTriangleRectangleIsocele;
  		ch = getString("triangleRectangleIsocele");
  	}
  	else
    if (radiobtnTriangleEquilateral.isSelected()) {
    	proto = frame.constructionTriangleEquilateral;
  		ch = getString("triangleEquilateral");
    }
  	else
  	if (radiobtnCarre.isSelected()) {
  		proto = frame.constructionCarre;
  		ch = getString("carre");  		
  	}
  	else
  	if (radiobtnRectangle.isSelected()) {
  		proto = frame.constructionRectangle;
  		ch = getString("rectangle");
  	}
  	else
  	if (radiobtnParallelogramme.isSelected()) {
  		proto = frame.constructionParallelogramme;
  		ch = getString("parallelogramme");
  	}
  	else
  	if (radiobtnLosange.isSelected()) {
  		proto = frame.constructionLosange;
  		ch = getString("losange");
  	}
  	else
   	if (radiobtnTrapeze.isSelected()) {
   		proto = frame.constructionTrapeze;
  		ch = getString("trapeze");
   	}
  	else
  	if (radiobtnTrapezeRectangle.isSelected()) {
  		proto = frame.constructionTrapezeRectangle;
  		ch = getString("trapezeRectangle");
  	}
  	else
  	if (radiobtnTrapezeIsocele.isSelected()) {
  		proto = frame.constructionTrapezeIsocele;
  		ch = getString("trapezeIsocele");
  	}
  	else
  	if (radiobtnHexagoneRegulier.isSelected()) {
  		proto = frame.constructionHexagoneRegulier;
  		ch = getString("hexagoneRegulier");
  	}
  	if (radiobtnPentagoneRegulier.isSelected()) {
  		proto = frame.constructionPentagoneRegulier;
  		ch = getString("pentagoneRegulier");
  	}
  	
		CMathGrafDoc doc = new CMathGrafDoc(frame);
		doc.listePrincipale.ajouteConstantePi();
		doc.listePrincipale.ajouteSegmentLongueurUnite();
		doc.listePrincipale.uniteAngle = frame.pref_angleUnit;
		MtgInternalFrame cadre = new MtgInternalFrame(frame, "MathGraph"
				+ Integer.toString(frame.instanceNouvelleFenetre()), doc);
		cadre.initialiseListes();
		try {
			cadre.gestionnaire.initialise();
		} catch (IOException e) {
		}
		// frame.metAJourIconesEtMenus(cadre.document);  // Version 4.1.2 : Inutile car fait dans MtgInternalFrame.internalFrameActivated
		frame.mtgDesktopPane.add(cadre);
		// On cre un item de menu dans le menu Fenetres pour activer cete fentre
		MtgMenuFileItem item = new MtgMenuFileItem(cadre);
		frame.menuFenetre.add(item);  	
  	CListeObjets liste = doc.listePrincipale;
		CImplementationProto impProto = new CImplementationProto(liste, proto);
		impProto.implemente(liste, proto, cadre);
		impProto.nomProto = ch;
		// Modification version 3.5.1
		frame.mtgDesktopPane.getDesktopManager().activateFrame(cadre);
		// La ligne suivante est ncessaire pour Linux car sinon MtgInternalFrame.internalFrameActivated n'est pas appel
		// et la figure n'est visible qu'aprs un clic ou une action sur la figure
    try {
      cadre.setSelected(true);
    } catch (java.beans.PropertyVetoException err) {};			
		if (maximise) {
			try {
				cadre.setMaximum(true);
			} catch (PropertyVetoException e1) {
				frame.mtgDesktopPane.getDesktopManager().maximizeFrame(cadre);
			}
		}
		else {
			Dimension sizefen = new Dimension(cadre.getWidth(), cadre.getHeight());
			frame.preparePositionPourFenenetre(sizefen);
			cadre.setLocation(frame.xfenetre, frame.yfenetre);
		}
 	
		frame.activeOutilCapture();
    dispose();
    setRetourOK(true);
  }       
}
