/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Dimension;
import java.awt.FlowLayout;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JLabel;
import javax.swing.JPanel;

import objets.CCalculBase;
import objets.CFoncComplexe;
import objets.CListeObjets;
import types.Int;

import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;

@SuppressWarnings("serial")
public class FonctionComplexeDlg extends MtgDlg {
	private CFoncComplexe fonctionAModifier;
	private boolean modification;
  private javax.swing.JButton btnCancel;
  private javax.swing.JButton btnOK;
  private JLabel label1;
  private JLabel label2;
  private JLabel label3;
  private javax.swing.JTextField editeurFormule;
  private javax.swing.JTextField editeurNom;
  private javax.swing.JTextField editeurVariableFormelle;
  private PaneBoutonsListeValeursEtFonctionsTextField paneListeValeurEtFonctions;  

  public FonctionComplexeDlg(MtgFrame frameInit, CFoncComplexe fonctionAModifierInit, boolean modificationInit) {
    super (getString("fonctionComplexeDialog1"), true, frameInit);
    fonctionAModifier = fonctionAModifierInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }
  private void initComponents() {
    btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnOKActionPerformed(evt);
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnCancelActionPerformed(evt);
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());
    label1 = new JLabel(getString("fonctionDialog2"), JLabel.RIGHT);
    label1.setRequestFocusEnabled(false);
    label2 = new JLabel(getString("fonctionDialog3"), JLabel.RIGHT);
    label2.setRequestFocusEnabled(false);
    label3 = new JLabel(getString("fonctionDialog4"), JLabel.RIGHT);
    label3.setRequestFocusEnabled(false);
    editeurFormule = new javax.swing.JTextField(null, 30);
    editeurFormule.setAutoscrolls(true);
    editeurNom = new javax.swing.JTextField(null,8);
    editeurVariableFormelle = new javax.swing.JTextField(null,8);
  	CListeObjets liste = frame.cadreActif().listePr;
  	int indmax;
  	if (modification) indmax = liste.indexOf(fonctionAModifier) - 1;
  	else indmax = liste.size() - 1;
    paneListeValeurEtFonctions = new PaneBoutonsListeValeursEtFonctionsTextField(frame, editeurFormule, false, indmax, false);
    if (modification) {
    	editeurVariableFormelle.setText(fonctionAModifier.chaineVariable);
    	String[] ch = new String[1];
    	ch[0] = fonctionAModifier.chaineVariable;    	    	
      editeurFormule.setText(fonctionAModifier.calcul.chaineCalculSansPar(ch));
      editeurNom.setText(fonctionAModifier.nom());
      editeurNom.setEditable(false);
    }
    JPanel panePrincipal = new JPanel();
    JPanel paneLeft = new JPanel();
    JPanel paneRight = new JPanel();
    panePrincipal.setLayout(new FlowLayout());
    paneLeft.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.bottom = 5;
    constraints.insets.top = 5;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    
    setConstraints(constraints, 0, 0, 1, 1, 20, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(label1, constraints);
    
    setConstraints(constraints, 1, 0, 2, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    constraints.insets.bottom = 0;
    paneLeft.add(editeurNom, constraints);
    
    setConstraints(constraints, 0, 1, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label2, constraints);
   
    setConstraints(constraints, 1, 1, 2, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(editeurVariableFormelle, constraints);
    
    setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.NONE, GridBagConstraints.EAST);
    paneLeft.add(label3, constraints);
   
    setConstraints(constraints, 1, 2, 2, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    paneLeft.add(editeurFormule, constraints);
    
    constraints.insets.bottom = 5;
    setConstraints(constraints, 1, 3, 1, 1, 30, 20, GridBagConstraints.NONE, GridBagConstraints.WEST);
    paneLeft.add(paneListeValeurEtFonctions, constraints);
    
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
    panePrincipal.add(paneLeft);
    panePrincipal.add(paneRight);
    setContentPane(panePrincipal);
    pack();
  }
  
  private void btnOKActionPerformed(java.awt.event.ActionEvent evt) {                                      
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets listePr = cadre.listePr;
    String chNom = editeurNom.getText();
    String chVarFor = editeurVariableFormelle.getText();
    // On regarde d'abord si le nom propos est correct et non vide en cas de cration d'un calcul
  	// On vrifie d'abord la validit du nom de la fonction
    if (!modification && !listePr.validationNomCalcul(frame, chNom)) {
      editeurNom.requestFocus();
      return;
    }
    // puis celui de la variable formelle
    if (!listePr.validationNomVariableFormelle(frame, chVarFor)) {
      editeurVariableFormelle.requestFocus();
      return;
    }
    // On vrifie ensuite que le nom de la variable formelle n'est pas le mme que le nom de la fonction
    if (chNom.equalsIgnoreCase(chVarFor)) {
      dialogueAvertissement(getString("ch92"));
      editeurVariableFormelle.requestFocus();
      return;	    		    	
    }
    //Le nom est correct. On vrifie maintenant la formule
    String chCalcul = editeurFormule.getText();
    int indDernier; // Dernier objet autoris dans la formule
    if (modification) indDernier = listePr.indexOf(fonctionAModifier) - 1;
    else indDernier = listePr.size()-1;
    Int indErr = new Int();
    // On n'autorise d'utiliser que les objets crs par l'utilisateur
    String[] varFor = new String[1];
    varFor[0] = chVarFor;
    if (CCalculBase.verifieSyntaxeComplexe(listePr, chCalcul, indErr, 
        indDernier, varFor)) {
    	varFor[0] = chVarFor;
      CCalculBase calc = CCalculBase.creeCalculBaseComplexe(chCalcul ,listePr, 0,
              chCalcul.length()-1, varFor);
      fonctionAModifier.calcul = calc;
      fonctionAModifier.chaineVariable = chVarFor;
      
      // Modification version 4.5.2 : Quand la chane de calcul sera reconstruite elle pourra tre diffrente de celle ici entre
      // On la change tout de suit
      fonctionAModifier.chaineCalcul = calc.chaineCalculSansPar(varFor);      
      
      if (!modification) fonctionAModifier.nomCalcul = chNom;
      if (modification) cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil48"));
      setRetourOK(true);
      dispose();
    }
    else {
    	dialogueAvertissement(getString("ch5"));
      editeurFormule.setCaretPosition(indErr.getValue());
      editeurFormule.requestFocus();
      return;     
    }
  }                                     

  private void btnCancelActionPerformed(java.awt.event.ActionEvent evt) {                                          
    dispose();
  }     
}
