/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.Insets;
import java.awt.Point;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.event.KeyEvent;
import java.awt.event.KeyListener;
import java.util.Vector;

import javax.swing.BorderFactory;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JToolBar;
import javax.swing.event.ListSelectionEvent;
import javax.swing.event.ListSelectionListener;
import mathgraph32.MtgDesktopPane;
import mathgraph32.MtgFrame;
import objets.CLatex;
import objets.CListeObjets;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class LatexDlg extends MtgDlg {
	private boolean modification; // True si on modifie les coordonnes d'un commentaire dj cr
	private CLatex latex;
	private CLatex latexClignotant;
	private String codeLatexInitial; // Le code initial pour le remettre  jour au cas o on sort par chappement
	private int indiceMax;
	private JLabel label1;
	private javax.swing.JButton btnInsertionAffichageDynamique;
  private javax.swing.JButton btnCancel;
  private javax.swing.JButton btnOK;
  private PaneTaillePolice paneTaillePolice;
  private PaneModeAffichage paneModeAffichage;
  private TextAreaLatexAvecCarSpeciaux editeurTexteLatex;
  private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

	public LatexDlg(MtgFrame frameInit, CLatex latexInit, boolean modificationInit,
			CLatex latexClignotantInit)	{
		super(getString("dlgLatex1"), true, frameInit);
		latex = latexInit;
		latexClignotant = latexClignotantInit;
		modification = modificationInit;
    CListeObjets liste = frame.cadreActif().listePr;
    // Dans le cas ou on cre un nouvel objet, celui-ci n'a pas encore t ajout
    //  la liste des objets de l'applet.
    if (modification) indiceMax = liste.indexOf(latexInit) - 1;
    else indiceMax = liste.size() - 1;
    
 		editeurTexteLatex = new TextAreaLatexAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		// editeurTexteLatex.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme      
 		editeurTexteLatex.setRows(8);
 		editeurTexteLatex.setColumns(45);
    JScrollPane scrollEditeurTexte = new JScrollPane
  		(editeurTexteLatex, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    if (modification) {
    	editeurTexteLatex.setText(latex.chaineCommentaire);
    	codeLatexInitial = latex.chaineCommentaire;
    }
    
    // On prpare les boutons spciaux latex et les deux barre qui les contiendront
		JToolBar bar1 = new JToolBar(JToolBar.HORIZONTAL);
		bar1.setBorder(BorderFactory.createEmptyBorder());	
		MtgLatexButton buttonQuotient = new MtgLatexButton("LatexQuotient", "\\frac{ }{ }", 6, editeurTexteLatex);
		bar1.add(buttonQuotient);
		MtgLatexButton buttonRacine = new MtgLatexButton("LatexRacine", "\\sqrt{}", 6, editeurTexteLatex);
		bar1.add(buttonRacine);
		MtgLatexButton buttonRacineN = new MtgLatexButton("LatexRacineN", "\\sqrt [ ] { }", 7, editeurTexteLatex);
		bar1.add(buttonRacineN);
		MtgLatexButton buttonVecteur = new MtgLatexButton("LatexVecteur", "\\overrightarrow { }", 17, editeurTexteLatex);
		bar1.add(buttonVecteur);
		MtgLatexButton buttonMesureAlgebrique = new MtgLatexButton("LatexMesureAlgebrique", "\\overline { }", 11, editeurTexteLatex);
		bar1.add(buttonMesureAlgebrique);
		MtgLatexButton buttonSousligne = new MtgLatexButton("LatexSouligne", "\\underline { }", 12, editeurTexteLatex);
		bar1.add(buttonSousligne);
		MtgLatexButton buttonIntegrale = new MtgLatexButton("LatexIntegrale", "\\int_{ }^{ } ", 6, editeurTexteLatex);
		bar1.add(buttonIntegrale);
		MtgLatexButton buttonSigma = new MtgLatexButton("LatexSigma", "\\sum_{ }^{ }", 6, editeurTexteLatex);
		bar1.add(buttonSigma);
		JToolBar bar2 = new JToolBar(JToolBar.HORIZONTAL);		
		bar2.setBorder(BorderFactory.createEmptyBorder());	
		MtgLatexButton buttonParouv = new MtgLatexButton("LatexPar", "\\left( { } \\right)", 8, editeurTexteLatex);
		bar2.add(buttonParouv);
		MtgLatexButton buttonCrochet = new MtgLatexButton("LatexCrochet", "\\left[ { } \\right]", 8, editeurTexteLatex);
		bar2.add(buttonCrochet);
		MtgLatexButton buttonAccolade = new MtgLatexButton("LatexAccolade", "\\left\\{ { } \\right.", 9, editeurTexteLatex);
		bar2.add(buttonAccolade);
		MtgLatexButton buttonAccoladeDouble = new MtgLatexButton("LatexAccoladeDouble", "\\left\\{ { } \\right}", 9, editeurTexteLatex);
		bar2.add(buttonAccoladeDouble);
		MtgLatexButton buttonNorme = new MtgLatexButton("LatexNorme", "\\left\\| { } \\right\\|", 9, editeurTexteLatex);
		bar2.add(buttonNorme);
		// On met un array align  gauche pour une matrice colonne
		MtgLatexButton buttonMatrice21 = new MtgLatexButton("LatexMatrice21", "\\begin{array}{l}\n{ }\n\\\\ { }\n\\end{array}", 18, editeurTexteLatex);
		bar2.add(buttonMatrice21);
		MtgLatexButton buttonMatrice12 = new MtgLatexButton("LatexMatrice12", "\\begin{matrix}\n{ } & { } \n\\end{matrix}", 16, editeurTexteLatex);
		bar2.add(buttonMatrice12);
		MtgLatexButton buttonMatrice22 = new MtgLatexButton("LatexMatrice22", "\\begin{matrix}\n{ } & { } \n\\\\ { } & { } \n\\end{matrix}", 16, editeurTexteLatex);
		bar2.add(buttonMatrice22);
		JToolBar bar3 = new JToolBar(JToolBar.HORIZONTAL);		
		bar3.setBorder(BorderFactory.createEmptyBorder());	
		MtgLatexButton buttonAngle = new MtgLatexButton("LatexAngle", "\\widehat { }", 10, editeurTexteLatex);
		bar3.add(buttonAngle);
		MtgLatexButton buttonLimite = new MtgLatexButton("LatexLimite", "\\lim_{x \\to { }}", 13, editeurTexteLatex);
		bar3.add(buttonLimite);
		MtgLatexButton buttonSin = new MtgLatexButton("LatexSin", "\\sin ", 5, editeurTexteLatex); // Modifi version 4.5.2
		bar3.add(buttonSin);
		MtgLatexButton buttonCos = new MtgLatexButton("LatexCos", "\\cos ", 5, editeurTexteLatex); // Modifi version 4.5.2
		bar3.add(buttonCos);
		MtgLatexButton buttonTan = new MtgLatexButton("LatexTan", "\\tan ", 5, editeurTexteLatex); // Modifi version 4.5.2
		bar3.add(buttonTan);
		MtgLatexButton buttonExp = new MtgLatexButton("LatexExp", "\\mathrm{e}^{ }", 12, editeurTexteLatex);
		bar3.add(buttonExp);
		MtgLatexButton buttonLn = new MtgLatexButton("LatexLn", "\\ln ", 4, editeurTexteLatex); // Modifi version 4.5.2
		bar3.add(buttonLn);
		MtgLatexButton buttonTexte = new MtgLatexButton("LatexTexte", "\\text{}", 6, editeurTexteLatex);
		bar3.add(buttonTexte);

 		label1 = new JLabel(getString("dlgLatex2"));
 		btnOK = new javax.swing.JButton(getString("btnOK"));
    btnOK.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnOKActionPerformed();
      }
    });    
    btnCancel = new javax.swing.JButton(getString("btnCancel"));
    btnCancel.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(ActionEvent evt) {
        btnCancelActionPerformed();
      }
    });
    btnOK.setMaximumSize(btnCancel.getPreferredSize());    paneTaillePolice = new PaneTaillePolice(latexInit.indiceTaillePolice);
    paneTaillePolice.listeTailles.addListSelectionListener(new ListSelectionListener() {
			public void valueChanged(ListSelectionEvent e) {
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage = new PaneModeAffichage(frame, latexInit, true);
    paneModeAffichage.checkEffacementFond.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage.radioEncadrement3D.addActionListener(new ActionListener() {
			public void actionPerformed(ActionEvent e) {
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage.radioEncadrementSimple.addActionListener(new ActionListener() {	
			public void actionPerformed(ActionEvent e) {
				metAJourLatexClignotant();	
			}
		});
    paneModeAffichage.radioSansEncadrement.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				metAJourLatexClignotant();
			}
		});
    // Ajout version 4.7.1.1
    paneModeAffichage.rbAligHorCentre.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				latexClignotant.dec_x = 0;
				latexClignotant.dec_y = 0;
				//
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage.rbAligHorDroit.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				latexClignotant.dec_x = 0;
				latexClignotant.dec_y = 0;
				//
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage.rbAligHorGauche.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				latexClignotant.dec_x = 0;
				latexClignotant.dec_y = 0;
				//
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage.rbAligVertBas.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				latexClignotant.dec_x = 0;
				latexClignotant.dec_y = 0;
				//
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage.rbAligVertCentre.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				latexClignotant.dec_x = 0;
				latexClignotant.dec_y = 0;
				//
				metAJourLatexClignotant();
			}
		});
    paneModeAffichage.rbAligVertHaut.addActionListener(new ActionListener() {			
			public void actionPerformed(ActionEvent e) {
				// Ajout version 4.7.7
				latexClignotant.dec_x = 0;
				latexClignotant.dec_y = 0;
				//
				metAJourLatexClignotant();
			}
		});
    
		// Fin ajout
 		editeurTexteLatex.addKeyListener(new KeyListener() {
			public void keyTyped(KeyEvent e) {
				// TODO Auto-generated method stub
			}
			
			public void keyReleased(KeyEvent e) {
				// Seul keyReleased tient compte du dernier caractre press
				metAJourLatexClignotant();
			}
			public void keyPressed(KeyEvent e) {
				// TODO Auto-generated method stub
				
			}
		});
    paneModeAffichage.colorButton.associeA(latexClignotant);

 		btnInsertionAffichageDynamique = new JButton(getString("btnInsertionValeurDynamique"));
 		btnInsertionAffichageDynamique.addActionListener(new ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnInsertionAffichageDynamiqueActionPerformed();
      }
 		});
 		
    JPanel pane = new JPanel();
    GridBagLayout gridBag = new GridBagLayout();    
    pane.setLayout(gridBag);
    GridBagConstraints constraints = new GridBagConstraints();
    constraints.insets.top = 4;
    constraints.insets.bottom = 4;
    constraints.insets.left = 5;
    constraints.insets.right = 5;
    setConstraints(constraints, 0, 0, 2, 1, 80, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(bar1, constraints);
    setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(bar2, constraints);
    setConstraints(constraints, 0, 2, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(bar3, constraints);
    setConstraints(constraints, 0, 3, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(label1, constraints);
    constraints.insets.top = 2;
    setConstraints(constraints, 0, 4, 2, 1, 0, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
    pane.add(scrollEditeurTexte, constraints);
    constraints.insets.top = 4;
    setConstraints(constraints, 0, 5, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
    pane.add(btnInsertionAffichageDynamique, constraints);
    setConstraints(constraints, 0, 6, 1, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneTaillePolice, constraints);
    setConstraints(constraints, 1, 6, 2, 1, 0, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
    pane.add(paneModeAffichage, constraints);
    JPanel paneRightHigh = new JPanel();
    paneRightHigh.setLayout(new BoxLayout(paneRightHigh, BoxLayout.Y_AXIS));
    paneRightHigh.add(btnOK);
		paneRightHigh.add(Box.createRigidArea(new Dimension(0, 5))); 		
    paneRightHigh.add(btnCancel);
    setConstraints(constraints, 2, 0, 1, 4, 20, 50, GridBagConstraints.NONE, GridBagConstraints.NORTHEAST);
    pane.add(paneRightHigh, constraints);
    setContentPane(pane);
    getRootPane().setDefaultButton(btnOK);    

    // On change l'ordre par dfaut des composants
    Vector<Component> order = new Vector<Component>(3);
    order.add(editeurTexteLatex);
    order.add(btnOK);
    order.add(btnCancel);
    mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
    setFocusTraversalPolicy(mtgFocusTraversalPolicy);
    editeurTexteLatex.requestFocus();
    pack();
	}

  private void btnCancelActionPerformed() {
  	if (modification) {
  		latex.chaineCommentaire = codeLatexInitial;
			latex.creeFormule();
			latex.determineDependances();
			latex.positionne(false, frame.cadreActif().dimf);
  	}
  	
  	dispose();
  }

  private void btnOKActionPerformed() {  
  	if (editeurTexteLatex.getText().length() == 0) {
  		dialogueAvertissement(getString("dlgCommentaire3"));
  		editeurTexteLatex.requestFocus();
  	}
  	else {
  		latex.chaineCommentaire = editeurTexteLatex.getText();
  		latex.indiceTaillePolice = paneTaillePolice.indiceTaille();
  		latex.encadrement = paneModeAffichage.styleEncadrement();
  		latex.effacementFond = paneModeAffichage.effacementFond();
  		latex.couleurFondAffichage = paneModeAffichage.couleurFond();
  		// Ajouts version 4.5
  		latex.alignementHorizontal = paneModeAffichage.alignementHorizontal();
  		latex.alignementVertical = paneModeAffichage.alignementVertical();
  		//
  		// Ajout version 4.7.7
  		latex.dec_x = latexClignotant.dec_x;
  		latex.dec_y = latexClignotant.dec_y;  		
  		//
  		frame.pref_IndiceTaillePoliceAffichages = latex.indiceTaillePolice;
  		frame.cadreActif().couleurFondAffichage = latex.couleurFondAffichage;
  		latex.dejaPositionne = false;
  		latex.determineDependances();
  		// latex.creeFormule(); // Supprim version 4.7.4
  		setRetourOK(true);
  		dispose();
  	}
  }
  
  private void btnInsertionAffichageDynamiqueActionPerformed() {
  	InsertionDynamiqueDansLatexDlg dlg = new InsertionDynamiqueDansLatexDlg(frame, indiceMax, editeurTexteLatex);
  	dlg.setLocationRelativeTo(this);
  	dlg.setVisible(true);
  }
  
  private void metAJourLatexClignotant() {
		latexClignotant.chaineCommentaire = editeurTexteLatex.getText();
		latexClignotant.traiteCodesLatex();
		latexClignotant.indiceTaillePolice = paneTaillePolice.indiceTaille();
		latexClignotant.encadrement = paneModeAffichage.styleEncadrement();
		latexClignotant.effacementFond = paneModeAffichage.effacementFond();
		// Ajout version 4.7.1.1
		latexClignotant.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		latexClignotant.alignementVertical = paneModeAffichage.alignementVertical();
		// Fin ajout
		latexClignotant.determineDependances();
		latexClignotant.creeFormule();
		latexClignotant.positionne(false, frame.cadreActif().dimf);
		frame.cadreActif().redessineSansCalcul();  	
  }
  
  
  /**
   * Version 3.2.9 : on redfinit setVisible pour que le dialogue se positionne automatiquement
   * en bas et  droite de la fentre principale.
   */
  @Override
  public void setVisible(boolean b) {
  	MtgDesktopPane mtgdp = frame.mtgDesktopPane;
  	Dimension s = mtgdp.getSize();
  	Dimension d = getSize();
  	// Point p = mtgdp.getLocation(); Modifi version 4.5.1
  	Point p = mtgdp.getLocationOnScreen();
  	Insets insets = getInsets();
  	setLocation(p .x + s.width - d.width - insets.right, p.y + s.height - d.height - insets.bottom);
  	super.setVisible(b);
  }
  
  @Override
  public void metAJourAffichageClignotant() {
  	metAJourLatexClignotant();
  }
  
	public boolean estCentreSurFenetre() {
		return false;
	}
  
}
