/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.io.File;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.util.Vector;

import javax.sound.sampled.AudioInputStream;
import javax.sound.sampled.AudioSystem;
import javax.sound.sampled.Clip;
import javax.sound.sampled.LineUnavailableException;
import javax.sound.sampled.UnsupportedAudioFileException;
import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.JButton;
import javax.swing.JFileChooser;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import javax.swing.JTextField;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CListeObjets;
import objets.CMacroJouantSon;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroJouantSonDlg extends MtgDlg {
	private boolean modification;
	private CMacroJouantSon macro;
	private JLabel label1, label2, label3, label4, label5;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private JTextField editeurCheminAcces;
	private JButton btnParcourir, btnEcouter;
	private JTextArea editeurCommentaire;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private JButton btnOK, btnCancel;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;
  public MacroJouantSonDlg(MtgFrame frameInit, CMacroJouantSon macroInit, boolean modificationInit) {
    super (getString("macroJouantSonDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }
	
	public void initComponents() {
		label1 = new JLabel(getString("macroApparitionDlg2"));
		label2 = new JLabel(getString("macroApparitionDlg3"));
		label3 = new JLabel(getString("macroJouantSonDlg2"));
		label4 = new JLabel(getString("macroJouantSonDlg3"));
		label5 = new JLabel(getString("macroJouantSonDlg4"));
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurCommentaire = new JTextArea();
		// editeurCommentaire.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme      
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(4);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneCommentaire.setPreferredSize(new Dimension(300, 90));
    editeurCheminAcces = new JTextField(25);
		if (modification) {
			editeurIntitule.setText(macro.intitule);
			editeurCheminAcces.setText(macro.wavePath);
			editeurCommentaire.setText(macro.commentaireMacro);
			editeurCommentaire.setCaretPosition(0);
		}
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());
	  btnParcourir = new JButton(getString("parcourir"));
    btnParcourir.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnParcourirActionPerformed();
      }
    });
    btnEcouter = new JButton(getString("ecouter"));
    btnEcouter.addActionListener(new java.awt.event.ActionListener() {
      public void actionPerformed(java.awt.event.ActionEvent evt) {
        btnEcouterActionPerformed();
      }
    });
	  
	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);
	  
    JPanel panePrincipal = new JPanel();
    JPanel paneRight = new JPanel();
    JPanel paneMilieu = new JPanel();
    JPanel paneBas = new JPanel();
    paneMilieu.setLayout(new GridBagLayout());
    panePrincipal.setLayout(new GridBagLayout());
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);

		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 5;
		constraints.insets.top = 5;
		constraints.insets.left = 5;
		constraints.insets.right = 5;
		
		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIntitule, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		
		setConstraints(constraints, 0, 0, 1, 1, 30, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(label3, constraints);
		setConstraints(constraints, 1, 0, 2, 1, 70, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(editeurCheminAcces, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 35, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(btnParcourir, constraints);
		setConstraints(constraints, 2, 1, 2, 1, 35, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(btnEcouter, constraints);
		
		setConstraints(constraints, 0, 3, 3, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.NORTHWEST);
		panePrincipal.add(paneMilieu, constraints);
		
		paneBas.add(paneTaillePolice);
		paneBas.add(paneModeAffichage);
		setConstraints(constraints, 0, 4, 2, 1, 0, 30, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);
		constraints.insets.bottom = 0;
		constraints.insets.top = 0;
		setConstraints(constraints, 0, 5, 3, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(label4, constraints);
		setConstraints(constraints, 0, 6, 3, 1, 0, 10, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(label5, constraints);		
   
		setContentPane(panePrincipal);
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(7);
		order.add(editeurIntitule);
		order.add(editeurCommentaire);
		order.add(editeurCheminAcces);
		order.add(paneTaillePolice.listeTailles);
		order.add(paneModeAffichage.effacementFond);
		order.add(btnOK);
		order.add(btnCancel);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();
	}

  private void btnOKActionPerformed() {
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	// On vrifie si le chemin d'accs au fichier sonore est valide
		MtgInternalFrame cadre = frame.cadreActif();
		String chRacine = cadre.document.cheminRacine();
		String ch = chRacine + editeurCheminAcces.getText();
		FileReader fr;
		try {
			fr = new FileReader(ch);
			fr.close();
		} 
		catch (FileNotFoundException e3) {
			dialogueAvertissement(getString("ch49"));
			editeurCheminAcces.requestFocus();
			return;
		} 
		catch (IOException e2) {
			dialogueAvertissement(getString("ch49"));
			editeurCheminAcces.requestFocus();
			return;
		}
  	macro.intitule = chintitule;
  	macro.commentaireMacro = editeurCommentaire.getText();
  	macro.wavePath = editeurCheminAcces.getText();
  	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
  	macro.effacementFond = paneModeAffichage.effacementFond();
  	macro.couleurFondAffichage = paneModeAffichage.couleurFond();
		// Ajouts version 4.5
		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		macro.alignementVertical = paneModeAffichage.alignementVertical();
		//
		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
		macro.prepareClip(liste);
  	setRetourOK(true);
  	dispose(); 	
  }
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  private void btnParcourirActionPerformed() {
		class MyFilter extends javax.swing.filechooser.FileFilter {
			public boolean accept(File file) {
				if (file.isDirectory())
					return true;
				String filename = file.getName().toLowerCase();
				return filename.endsWith(".wav") || filename.endsWith(".au") || filename.endsWith(".aiff") ;
			}
			public String getDescription() {
				return "*.wav; *.au; *.aiff";
			}
		}
		MtgInternalFrame cadre = frame.cadreActif();
		JFileChooser fc = new JFileChooser(frame.repertoireFichiers);
		fc.setApproveButtonText(getString("btnChoisir"));
		fc.setFileFilter(new MyFilter());
		fc.showOpenDialog(frame);
		File file = fc.getSelectedFile();
		if (file != null) {
			String ch = file.getAbsolutePath();
			// On regarde si il s'agit d'un sous chemin du chemin d'accs au document
			String chRacine = cadre.document.cheminRacine();
			int ind = ch.indexOf(chRacine);
			if (ind == -1) {
				dialogueAvertissement(getString("ch47"));
			}
			else editeurCheminAcces.setText(ch.substring(chRacine.length()));
		}
  }
  
  private void btnEcouterActionPerformed() {
  	MtgInternalFrame cadre = frame.cadreActif();
  	String ch = cadre.document.cheminRacine() + editeurCheminAcces.getText();
  	File file = new File(ch);
    AudioInputStream audioInputStream;
		try {
			audioInputStream = AudioSystem.getAudioInputStream(file);
	  	// On charge les donnes sonores dans clip
	  	Clip clip = AudioSystem.getClip();
	  	clip.open(audioInputStream); 
	    clip.setFramePosition(0);
	    clip.start();
		}
  	catch (UnsupportedAudioFileException e) {
  		dialogueAvertissement(getString("ch48"));
		} 
  	catch (IOException e) {
  		dialogueAvertissement(getString("ch48"));
		}
  	catch (LineUnavailableException e) {
  		dialogueAvertissement(getString("ch48"));
		}
  }
	
}
