/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package dialogs;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseListener;
import java.net.URL;
import java.util.Vector;

import javax.swing.Box;
import javax.swing.BoxLayout;
import javax.swing.ImageIcon;
import javax.swing.JButton;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JTextArea;
import mathgraph32.MtgFrame;
import objets.CListeObjets;
import objets.CMacro;
import objets.CMacroSuiteMacros;
import types.NatureCarSpe;

@SuppressWarnings("serial")
public class MacroSuiteMacrosDlg extends MtgDlg {
	private boolean modification;
	private CMacroSuiteMacros macro;
	private JLabel label1, label2, label3, label4;
	private TextFieldAvecCarSpeciaux	editeurIntitule;
	private JTextArea editeurCommentaire;
	private ListeMacros listeMacros1;
	private JScrollPane scrollPaneMacros1;
	private ListeMacros listeMacros2;
	private JScrollPane scrollPaneMacros2;
	private PaneTaillePolice paneTaillePolice;
	private PaneModeAffichagePourMacro paneModeAffichage;
	private JButton btnOK, btnCancel;
	private JButton btnInserer, btnInsererFin, btnInsererDebut, btnRetirer, btnRetirerTout;
	private JButton btnHaut;
	private JButton btnBas;
	private MtgFocusTraversalPolicy mtgFocusTraversalPolicy;

  public MacroSuiteMacrosDlg(MtgFrame frameInit, CMacroSuiteMacros macroInit, boolean modificationInit) {
    super (getString("macroSuiteMacrosDlg1"), true, frameInit);
    macro = macroInit;
    modification = modificationInit;
    initComponents();
    getRootPane().setDefaultButton(btnOK);    
  }
	
  private void initComponents() {
		label1 = new JLabel(getString("macroApparitionDlg2"));
		label2 = new JLabel(getString("macroApparitionDlg3"));
		label3 = new JLabel(getString("macroSuiteMacrosDlg2"));
		label4 = new JLabel(getString("macroSuiteMacrosDlg3"));
		editeurIntitule = new TextFieldAvecCarSpeciaux(this, frame, NatureCarSpe.Tout);
		editeurCommentaire = new JTextArea();
		// editeurCommentaire.setFont(FonteNom.getFonteNom(2, FonteNom.Normal, -1)); // Supprim version 4.7.5.2 pris en charge augmentation polices systme      
		editeurCommentaire.setColumns(20);
		editeurCommentaire.setLineWrap(false);
		editeurCommentaire.setRows(4);
    JScrollPane scrollPaneCommentaire = new JScrollPane(editeurCommentaire,
    		JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED, JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED);
    scrollPaneCommentaire.setPreferredSize(new Dimension(300, 90));
    CListeObjets liste = frame.cadreActif().listePr;
    int indmax;
    // Correction bug version 4.7.2.2
    // if (modification) indmax = liste.indexOf(macro); else indmax = liste.size() - 1;
    if (modification) indmax = liste.indexOf(macro)-1; else indmax = liste.size() - 1;
    listeMacros1 = new ListeMacros(liste, indmax);
    listeMacros1.setFixedCellWidth(200); // 200 pixels de large pour les deux listes
    scrollPaneMacros1 = new JScrollPane(listeMacros1, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
    listeMacros2 = new ListeMacros(liste);
    listeMacros2.setFixedCellWidth(200);
    listeMacros1.addMouseListener(new MouseListener() {
			public void mouseClicked(MouseEvent evt) {
				listeMacros1MouseClicked(evt);
			}

			public void mouseEntered(MouseEvent evt) {
			}

			public void mouseExited(MouseEvent evt) {
			}

			public void mousePressed(MouseEvent evt) {
			}

			public void mouseReleased(MouseEvent evt) {
			}
    });
    scrollPaneMacros2 = new JScrollPane(listeMacros2, JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED,
    		JScrollPane.HORIZONTAL_SCROLLBAR_AS_NEEDED );
		if (modification) {
			editeurIntitule.setText(macro.intitule);
			editeurCommentaire.setText(macro.commentaireMacro);
			editeurCommentaire.setCaretPosition(0);
			listeMacros2.ajouteMacrosDe(macro);
			listeMacros2.setSelectedIndex(0);
		}
	  btnInserer = new javax.swing.JButton(getString("btnInserer"));
	  btnInserer.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnInsererActionPerformed();
	    }
	  });        
	  btnInsererDebut = new javax.swing.JButton(getString("btnInsererDebut"));
	  btnInsererDebut.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnInsererDebutActionPerformed();
	    }
	  });        
	  btnInsererFin = new javax.swing.JButton(getString("btnInsererFin"));
	  btnInsererFin.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnInsererFinActionPerformed();
	    }
	  });        
	  btnRetirer = new javax.swing.JButton(getString("btnRetirer"));
	  btnRetirer.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnRetirerActionPerformed();
	    }
	  });        
	  btnRetirerTout = new javax.swing.JButton(getString("btnRetirerTout"));
	  btnRetirerTout.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnRetirerToutActionPerformed();
	    }
	  });
	  btnOK = new javax.swing.JButton(getString("btnOK"));
	  btnOK.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnOKActionPerformed();
	    }
	  });    
	  btnCancel = new javax.swing.JButton(getString("btnCancel"));
	  btnCancel.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnCancelActionPerformed();
	    }
	  });
	  btnOK.setMaximumSize(btnCancel.getPreferredSize());
    URL adr = MtgFrame.class.getResource("images/Up16.gif");    
    btnHaut = new JButton(new ImageIcon(adr));
	  btnHaut.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnHautActionPerformed();
	    }
	  });	  
    adr = MtgFrame.class.getResource("images/Down16.gif");    
    btnBas = new JButton(new ImageIcon(adr));
	  btnBas.addActionListener(new java.awt.event.ActionListener() {
	    public void actionPerformed(java.awt.event.ActionEvent evt) {
	      btnBasActionPerformed();
	    }
	  });	  
	  paneTaillePolice = new PaneTaillePolice(macro.indiceTaillePolice);
	  paneModeAffichage = new PaneModeAffichagePourMacro(frame, macro);
	  
    JPanel panePrincipal = new JPanel();
    panePrincipal.setLayout(new GridBagLayout());
    JPanel paneRight = new JPanel();
    JPanel paneMilieu = new JPanel();  // Pour la liste des variables et l'info sur la variable slectionne
    JPanel paneBas = new JPanel();
    JPanel paneBoutonsHautBas = new JPanel();
    paneBoutonsHautBas.setLayout(new BoxLayout(paneBoutonsHautBas, BoxLayout.Y_AXIS));
    paneRight.setLayout(new BoxLayout(paneRight, BoxLayout.Y_AXIS));
    paneRight.add(btnOK);
    paneRight.add(Box.createRigidArea(new Dimension(0,5)));
    paneRight.add(btnCancel);
    
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 2;
		constraints.insets.left = 5;
		constraints.insets.right = 5;

		paneBoutonsHautBas.add(btnHaut);
		paneBoutonsHautBas.add(btnBas);
		// paneMilieu est le JPanel contenant les deux listes de macros et les boutons correspondants
		paneMilieu.setLayout(new GridBagLayout());
		setConstraints(constraints, 0, 0, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(label3, constraints);
		setConstraints(constraints, 0, 1, 1, 1, 0, 50, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneMilieu.add(scrollPaneMacros1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		paneMilieu.add(label4, constraints);
		setConstraints(constraints, 1, 1, 1, 1, 0, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		paneMilieu.add(scrollPaneMacros2, constraints);
		setConstraints(constraints, 0, 2, 1, 1, 0, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
		paneMilieu.add(btnInserer, constraints);
		setConstraints(constraints, 0, 3, 1, 1, 0, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
		paneMilieu.add(btnInsererDebut, constraints);
		setConstraints(constraints, 0, 4, 1, 1, 0, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
		paneMilieu.add(btnInsererFin, constraints);
		setConstraints(constraints, 1, 2, 1, 1, 0, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
		paneMilieu.add(btnRetirer, constraints);
		setConstraints(constraints, 1, 3, 1, 1, 0, 20, GridBagConstraints.HORIZONTAL, GridBagConstraints.CENTER);
		paneMilieu.add(btnRetirerTout, constraints);
		setConstraints(constraints, 2, 0, 1, 2, 10, 0, GridBagConstraints.NONE, GridBagConstraints.CENTER);
		paneMilieu.add(paneBoutonsHautBas, constraints);

		
		paneBas.add(paneTaillePolice);
		paneBas.add(paneModeAffichage);		

		setConstraints(constraints, 0, 0, 1, 1, 20, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label1, constraints);
		setConstraints(constraints, 1, 0, 1, 1, 60, 0, GridBagConstraints.HORIZONTAL, GridBagConstraints.WEST);
		panePrincipal.add(editeurIntitule, constraints);
		setConstraints(constraints, 0, 1, 2, 1, 0, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		panePrincipal.add(label2, constraints);
		setConstraints(constraints, 0, 2, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(scrollPaneCommentaire, constraints);
		setConstraints(constraints, 0, 3, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneMilieu, constraints);
		setConstraints(constraints, 0, 5, 2, 1, 0, 30, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneBas, constraints);
 		setConstraints(constraints, 2, 0, 1, 2, 20, 0, GridBagConstraints.BOTH, GridBagConstraints.WEST);
		panePrincipal.add(paneRight, constraints);

		setContentPane(panePrincipal);
		// On change l'ordre par dfaut des composants
		Vector<Component> order = new Vector<Component>(8);
		order.add(editeurIntitule);
		order.add(editeurCommentaire);
		order.add(listeMacros1);
		order.add(listeMacros2);
		order.add(paneTaillePolice.listeTailles);
		order.add(paneModeAffichage.effacementFond);
		order.add(btnOK);
		order.add(btnCancel);
		mtgFocusTraversalPolicy = new MtgFocusTraversalPolicy(order);
		setFocusTraversalPolicy(mtgFocusTraversalPolicy);
		pack();			
  }
  
  private void btnInsererActionPerformed() {
  	CMacro mac = listeMacros1.macroChoisie();
  	if (listeMacros2.getSelectedIndex() == -1) listeMacros2.insereMacroFin(mac);
  	else listeMacros2.insereMacro(mac);
  }
  
  private void btnInsererDebutActionPerformed() {
  	CMacro mac = listeMacros1.macroChoisie();
  	listeMacros2.insereMacroDebut(mac);
  	
  }

  private void btnInsererFinActionPerformed() {
  	CMacro mac = listeMacros1.macroChoisie();
  	listeMacros2.insereMacroFin(mac);
  }
  
  private void btnRetirerActionPerformed() {
  	listeMacros2.retireMacroSelectionnee();
  }
  
  private void btnRetirerToutActionPerformed() {
  	listeMacros2.retireTout();
  	listeMacros1.metAJourListe();
  }
  
  private void btnOKActionPerformed() {
  	String chintitule = editeurIntitule.getText();
  	if (chintitule.length() == 0) {
  		dialogueAvertissement(getString("ch115"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	CListeObjets liste = frame.cadreActif().listePr;
  	if (!liste.validationIntituleMacro(chintitule, macro)) {
  		dialogueAvertissement(getString("ch116"));
  		editeurIntitule.requestFocus();
  		return;
  	}
  	if (listeMacros2.pointeurs.size() == 0) {
  		dialogueAvertissement(getString("ch131"));
  		editeurIntitule.requestFocus();
  		return;  		
  	}
  	macro.intitule = chintitule;
  	macro.commentaireMacro = editeurCommentaire.getText();
  	// L'affectation de la valeur dynamique a dj t faite par editeurValeur
  	macro.listeAssociee = listeMacros2.creeSousListe();
  	macro.indiceTaillePolice = paneTaillePolice.indiceTaille();
  	macro.effacementFond = paneModeAffichage.effacementFond();
  	macro.couleurFondAffichage = paneModeAffichage.couleurFond();
		// Ajouts version 4.5
		macro.alignementHorizontal = paneModeAffichage.alignementHorizontal();
		macro.alignementVertical = paneModeAffichage.alignementVertical();
		//
		frame.pref_IndiceTaillePoliceAffichages = macro.indiceTaillePolice;
		frame.cadreActif().couleurFondAffichage = macro.couleurFondAffichage;
  	setRetourOK(true);
  	dispose();  	
  }
  
  public void btnHautActionPerformed() {
  	listeMacros2.decaleBasMacroSelectionnee();
  }
  
  public void btnBasActionPerformed() {
  	listeMacros2.decaleHautMacroSelectionnee();  	
  }
  
  
  private void btnCancelActionPerformed() {
  	dispose();
  }
  
  public void listeMacros1MouseClicked(MouseEvent evt) {
		if (evt.getClickCount() == 2) btnInsererFinActionPerformed() ;
  }
}
