/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.Insets;

import javax.swing.JComponent;
import javax.swing.JMenu;
import javax.swing.JSeparator;
import javax.swing.tree.DefaultMutableTreeNode;

import actions.MtgAction;

import objets.CMathGrafDoc;

@SuppressWarnings("serial")
public class MtgMenu extends JMenu {
	// Inset pour les items de menus
	public static final Insets insetMenuItem = new Insets(2, 3, 2, 3);
	
	public MtgMenu(String ch) {
		super(ch);
	}
	
	public MtgMenu(MtgMenu menuInit) {
		super(menuInit.getText());
		for (int i = 0; i < menuInit.getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) menuInit.getMenuComponent(i);
			if (comp instanceof MtgMenuItem) {
  			MtgMenuItem mtgitem = (MtgMenuItem) comp;
  			add (new MtgMenuItem((MtgAction) mtgitem.getAction()));
			}
			else {
				if (comp instanceof MtgMenu){
					add(new MtgMenu(((MtgMenu) comp)));
				}
				else { // On a forcment un Jseparator
					add (new JSeparator());
				}
			}
		}
	}

  public final boolean adapteADocument(CMathGrafDoc doc, CMathGrafDoc docNiveau) {
  	boolean auMoinsUnItemVisible = false;
  	boolean bSeparator = false;
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgMenuItem) {
  			MtgMenuItem mtgitem = (MtgMenuItem) comp;
  			boolean b1 = mtgitem.adapteADocument(doc, docNiveau);
  			auMoinsUnItemVisible = auMoinsUnItemVisible || b1;
  			bSeparator = !b1;
			}
			else {
				if (comp instanceof MtgMenu){
					boolean b2 = ((MtgMenu) comp).adapteADocument(doc, docNiveau);
					auMoinsUnItemVisible = auMoinsUnItemVisible || b2;
					bSeparator = !b2;
				}
				else { // On a forcment un Jseparator
					if (bSeparator) comp.setVisible(false);
					bSeparator = true;
				}
			}
		}
		if (auMoinsUnItemVisible) setVisible(true);
		else setVisible(false);
		return auMoinsUnItemVisible;
  }
  
  
  /**
   * Fonction rcursive adaptant le document aux items cochs ou non
   * @param doc
   * @return
   */
  public final void adapteDocument(CMathGrafDoc doc) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			mtgitem.adapteDocument(doc);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).adapteDocument(doc);
				}
			}
		}
  }
  
  public final void ajouteTousItemsADocument(CMathGrafDoc doc) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
				doc.addItem(mtgitem.actionAssociee.idMenu());
  			mtgitem.adapteDocument(doc);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).ajouteTousItemsADocument(doc);
				}
			}
		}
  	
  }
  
  /**
   * Fonction slectionnant tous les items ayant une case  cocher. Modifi version 4.3
   */
  public final void selectAll(boolean bPreserveHorToolbar) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			mtgitem.setSelected(bPreserveHorToolbar ? !mtgitem.actionAssociee.estDansBarreHorizontale() : true);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).selectAll(bPreserveHorToolbar);
				}
			}
		} 	
  }
  
  /**
   * Fonction slectionnant tous les items ayant une case  cocher. MOdifi version 4.3
   */
  public final void deselectAll(boolean bPreserveHorToolbar) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			mtgitem.setSelected(bPreserveHorToolbar ? mtgitem.actionAssociee.estDansBarreHorizontale() : false);
			}
			else {
				if (comp instanceof MtgMenu){
					((MtgMenu) comp).deselectAll(bPreserveHorToolbar);
				}
			}
		} 	
  }
  
  
  public final void addCloneAvecCheck(MtgMenu cible, CMathGrafDoc docNiveau, CMathGrafDoc docReference) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if ((comp.isVisible() && comp instanceof MtgMenuItem)) {
				MtgMenuItem item = (MtgMenuItem) comp;
				MtgAction action = (MtgAction)item. getAction();
				boolean b = (docNiveau == null) ? true : docNiveau.itemPermis(action.idMenu());
				if (b && (action.idMenu() != 0)) {
					MtgCheckBoxMenuItem mtgCheckItem = new MtgCheckBoxMenuItem (item, docReference);
					cible.add(mtgCheckItem);
				}
 			}
			else {
				if (comp instanceof MtgMenu){
					if (comp.isVisible()) {
						MtgMenu menu = new MtgMenu(((MtgMenu) comp).getText());
						cible.add(menu);
						((MtgMenu) comp).addCloneAvecCheck(menu, docNiveau, docReference);
					}
				}
			}
		}
  }
  
  /**
   * Fonction renvoyant true si au moins un des item avec checkBox contenu dans le menu
   * ou un sous-menu a t modifi
   * @return
   */
  public final boolean modifie() {
  	boolean res = false;
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem mtgitem = (MtgCheckBoxMenuItem) comp;
  			res = res || mtgitem.modifie();
			}
			else {
				if (comp instanceof MtgMenu){
					res = res || ((MtgMenu) comp).modifie();
				}
			}
		}
  	return res;
  }
  
  public final void ajouteNoeud(DefaultMutableTreeNode node) {
		for (int i = 0; i < getMenuComponentCount(); i++) {
			JComponent comp = (JComponent) getMenuComponent(i);
			if (comp instanceof MtgCheckBoxMenuItem) {
				MtgCheckBoxMenuItem item = (MtgCheckBoxMenuItem) comp;
				item.ajouteNoeud(node);
 			}
			else {
				if (comp instanceof MtgMenu){
					MtgMenu menu = (MtgMenu) comp;
					DefaultMutableTreeNode newnode = new DefaultMutableTreeNode(menu.getText());
					node.add(newnode);
					menu.ajouteNoeud(newnode);
				}
			}
		}
  }
  
}
