/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package mathgraph32;

import java.awt.Color;
import java.awt.Dimension;
import java.awt.GridBagConstraints;
import java.awt.GridBagLayout;
import java.awt.event.MouseEvent;
import java.awt.event.MouseMotionListener;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JSlider;

import dialogs.MtgDlg;

@SuppressWarnings("serial")
public class MtgPaneTransparence extends JPanel {
	MtgFrame frame;
	private JLabel labelValeur;
	private JSlider slider;
	private float transparence;
	private MtgColorButton colorButton; // Bouton servant  choisir la couleur active
	// Le bouton charg de montrer l'effet du coefficient de transparence
	private MtgColorButtonTransparency colorButtonTransparency;	

	public MtgPaneTransparence(MtgFrame frameInit, float transparenceInit) {
    frame = frameInit;
    transparence = transparenceInit;
    initComponents();
    donneValeurCurseur(transparenceInit);
    metAJourBoutonTransparenceEtAffValeur();
	}
	
	public void initComponents() {	
		labelValeur = new JLabel();
		slider = new JSlider(5, 100, (int)(transparence*100));
		slider.setToolTipText(MtgFrame.getString("chaineTipCoefficientOpacite"));
		labelValeur.setToolTipText(MtgFrame.getString("chaineTipCoefficientOpacite"));
		slider.setMajorTickSpacing(10);
		slider.addMouseMotionListener(new MouseMotionListener() {      
			public void mouseDragged(MouseEvent arg0) {
				sliderMouseDragged();
			}

			public void mouseMoved(MouseEvent arg0) {	
			}
		});
		colorButton = new MtgColorButton(frame);
		colorButtonTransparency = new MtgColorButtonTransparency(frame, transparence);
		slider.setPreferredSize(new Dimension(55,15));
		setLayout(new GridBagLayout());
		GridBagConstraints constraints = new GridBagConstraints();
		constraints.insets.bottom = 0;
		constraints.insets.top = 3;
		constraints.insets.left = 0;
		constraints.insets.right = 0;
		MtgDlg.setConstraints(constraints, 0, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		add(colorButton, constraints);
		MtgDlg.setConstraints(constraints, 1, 0, 1, 1, 50, 0, GridBagConstraints.NONE, GridBagConstraints.WEST);
		add(colorButtonTransparency, constraints);
		MtgDlg.setConstraints(constraints, 0, 1, 1, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.WEST);
		add(slider, constraints);
		MtgDlg.setConstraints(constraints, 1, 1, 2, 1, 50, 10, GridBagConstraints.NONE, GridBagConstraints.CENTER);
		add(labelValeur, constraints);
		
	}
	
	private void sliderMouseDragged() {
		metAJourBoutonTransparenceEtAffValeur();
		MtgInternalFrame cadre = frame.cadreActif();
		if (cadre != null) {
			float newTrans = (float)(slider.getValue())/100;
			cadre.document.transparencePourSurfaces = newTrans;
			cadre.repaintFigureSansCalcul();
		}
	}
	
	public void donneValeurCurseur(float val) {
		slider.setValue((int) (val*100));
	}
	
	public Color couleurActive() {
		return colorButton.couleur;
	}
	
	public void donneCouleurActive(Color coul) {
    colorButton.couleur = coul;
    colorButton.repaint();	
	}
	/*
	 * Pour modifier la valeur du curseur et redessiner tout en prcisant le MtgInternalFrame qui vient d'tre activ
	 */
	public void donneValeurTransparence(float val, MtgInternalFrame cadre) {
		slider.setValue((int) (val*100));
		float newTrans = (float)(slider.getValue())/100;
		cadre.document.transparencePourSurfaces = newTrans;
		metAJourBoutonTransparenceEtAffValeur();
		// Quand MtgInternalFrame InternalFrameOpened est appel, le gbuffer est encore  null
		if (cadre.gbuffer != null) {
			cadre.repaintFigureSansCalcul();
		}
	}
	
	
	private void metAJourBoutonTransparenceEtAffValeur() {
		float newTrans = (float)(slider.getValue())/100;
		colorButtonTransparency.setTransparency(newTrans);
		colorButtonTransparency.repaint();
		labelValeur.setText(Integer.toString(slider.getValue()) + "%");
	}
	
	public void donneCouleurBoutonTransparence(Color color) {
		colorButtonTransparency.donneCouleur(color);
	}
	
}
