/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

// Cette classe doit mmoriser dans positionne() les valeurs de infoRandom et de dimfen
import java.io.*;
import java.awt.*;

import mathgraph32.MtgFrame;
import dialogs.GrapheSuiteRecurrenteReelleDlg;
import dialogs.MtgDlg;

import types.*;

import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

public class CGrapheSuiteRec extends CElementLigne {
	public CSuiteRec suiteAssociee;
	public CRepere repereAssocie;
	public boolean traitsDeRappelSurAbscisses;
	private CPointDansRepere point1;
	private CPointDansRepere point2; // Sert  tracer les lignes
	CSegment seg;
  private boolean infr;
  private DimensionFenetre dimf;

  /** Creates new CGrapheSuiteRec */
  public CGrapheSuiteRec() {
  }
    
  // Constructeur pour clonage
  public CGrapheSuiteRec(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean masqueInit,
      int styleInit, CSuiteRec suiteAssocieeInit, CRepere repereAssocieInit,
      boolean traitsDeRappelSurAbscissesInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleInit);
    suiteAssociee = suiteAssocieeInit;
    repereAssocie = repereAssocieInit;
    traitsDeRappelSurAbscisses = traitsDeRappelSurAbscissesInit;
    point1 = new CPointDansRepere();
    point1.rep = repereAssocie;
    point2 = new CPointDansRepere();
    point2.rep = repereAssocie;
    seg = new CSegment(liste, point1, point2);
  }
  
  /**
   * Lorsqu'on cre un graphe de suite rcurrente, le repre n'a pas encore t choisi.
   * Il faut aprs la cration appeler cette fonction pour que les points reprs utiliss de faon interne
   * soient initialiss
   * @param rep
   */
  public void associeARepere(CRepere rep) {
  	repereAssocie = rep;
  	point1.rep = rep;
  	point2.rep = rep;
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(suiteAssociee);
    int ind2 = listeSource.indexOf(repereAssocie);
    int ind3 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CGrapheSuiteRec(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, masque, style, (CSuiteRec)(listeCible.get(ind1, CSuiteRec.class)),
        (CRepere)(listeCible.get(ind2, CRepere.class)), traitsDeRappelSurAbscisses);
    return ptelb;
  }
   
  @Override
  public String chaineDesignation() {
    return getString("designationGrapheSuiteRec");
  }
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || suiteAssociee.dependDe(p) || repereAssocie.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || suiteAssociee.dependDePourReclassement(p) || repereAssocie.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this)) || suiteAssociee.dependDePourBoucle(p) || repereAssocie.dependDePourBoucle(p);
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
    existe = ((suiteAssociee.existe) && (repereAssocie.existe));
  // Cette classe doit mmoriser dans positionne() les valeurs de infoRandom et de dimfen
    infr = infoRandom;
    dimf = dimfen;
  }

  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {

    int styleTrait;
    if (masque) styleTrait = StyleTrait.traitPointille;
    else styleTrait = style;
    
    pDC.setStroke(Pinceau.CreePinceau(styleTrait, listeProprietaire));
    pDC.setColor(couleur);
    seg.donneCouleur(couleur);
    seg.donneStyle(style);
    for (int i = 1; i <= suiteAssociee.indiceDernierTermeExistant; i++)
    {
      if (i == 1)
      {
        point1.abs.donneValeur(suiteAssociee.valeurs[0]);
        point1.ord.donneValeur(suiteAssociee.valeurs[1]);
        point2.abs.donneValeur(suiteAssociee.valeurs[1]);
        point2.ord.donneValeur(suiteAssociee.valeurs[1]);
        point1.positionne(infr, dimf);
        point2.positionne(infr, dimf);
        seg.positionne(infr, dimf);
        if (seg.existe) seg.trace(pDC, couleurFond, transparence);
        if (traitsDeRappelSurAbscisses)
        {
          point1.abs.donneValeur(suiteAssociee.valeurs[0]);
          point1.ord.donneValeur(0);
          point2.abs.donneValeur(suiteAssociee.valeurs[0]);
          point2.ord.donneValeur(suiteAssociee.valeurs[1]);
          point1.positionne(infr, dimf);
          point2.positionne(infr, dimf);
          seg.positionne(infr, dimf);
          if (seg.existe) seg.trace(pDC, couleurFond, transparence);
        }
      }
      else
      {
        point1.abs.donneValeur(suiteAssociee.valeurs[i-1]);
        point1.ord.donneValeur(suiteAssociee.valeurs[i-1]);
        point2.abs.donneValeur(suiteAssociee.valeurs[i-1]);
        point2.ord.donneValeur(suiteAssociee.valeurs[i]);
        point1.positionne(infr, dimf);
        point2.positionne(infr, dimf);
        seg.positionne(infr, dimf);
        if (seg.existe) seg.trace(pDC, couleurFond, transparence);
        point1.abs.donneValeur(suiteAssociee.valeurs[i-1]);
        point1.ord.donneValeur(suiteAssociee.valeurs[i]);
        point2.abs.donneValeur(suiteAssociee.valeurs[i]);
        point2.ord.donneValeur(suiteAssociee.valeurs[i]);
        point1.positionne(infr, dimf);
        point2.positionne(infr, dimf);
        seg.positionne(infr, dimf);
        if (seg.existe) seg.trace(pDC, couleurFond, transparence);
      }
      if (traitsDeRappelSurAbscisses)
      {
        point1.abs.donneValeur(suiteAssociee.valeurs[i]);
        point1.ord.donneValeur(0);
        point2.abs.donneValeur(suiteAssociee.valeurs[i]);
        point2.ord.donneValeur(suiteAssociee.valeurs[i]);
        point1.positionne(infr, dimf);
        point2.positionne(infr, dimf);
        seg.positionne(infr, dimf);
        if (seg.existe) seg.trace(pDC, couleurFond, transparence);
      }
    }
  }
    
  @Override
  public boolean confonduAvec(CElementBase p) {
    CGrapheSuiteRec pt;
    if (p.getClass() == this.getClass()) {
      pt = (CGrapheSuiteRec) p;
   		return (repereAssocie == pt.repereAssocie) && (suiteAssociee == pt.suiteAssociee);
    }
    else return false;
  }
  
  @Override
  public String infoPourHistorique() {
  	return getString("chinfo88") + " " + suiteAssociee.nom() + "\n" +
  		getString("chinfo89") + " " + repereAssocie.nom();
  }
  
  @Override
  public boolean modifiableParMenu() {
  	// Modifi version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new GrapheSuiteRecurrenteReelleDlg(frame, this, true);
  }  
  
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	double dist;
  	int i;
  	
  	if (!existe || (masquage && masque)) return -1;
  	for (i = 1; i <= suiteAssociee.indiceDernierTermeExistant; i++) {
  		if (i == 1)	{
  			point1.abs.donneValeur(suiteAssociee.valeurs[0]);
  			point1.ord.donneValeur(suiteAssociee.valeurs[1]);
  			point2.abs.donneValeur(suiteAssociee.valeurs[1]);
  			point2.ord.donneValeur(suiteAssociee.valeurs[1]);
  			point1.positionne(false, dimf);
  			point2.positionne(false, dimf);
  			seg.positionne(false, dimf);
  			dist = seg.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  			if (traitsDeRappelSurAbscisses)
  			{
  				point1.abs.donneValeur(suiteAssociee.valeurs[0]);
  				point1.ord.donneValeur(0);
  				point2.abs.donneValeur(suiteAssociee.valeurs[0]);
  				point2.ord.donneValeur(suiteAssociee.valeurs[1]);
  				point1.positionne(false, dimf);
  				point2.positionne(false, dimf);
  				seg.positionne(false, dimf);
  				dist = seg.distancePoint(xp,yp,masquage);
    			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  			}
  		}
  		else
  		{
  			point1.abs.donneValeur(suiteAssociee.valeurs[i-1]);
  			point1.ord.donneValeur(suiteAssociee.valeurs[i-1]);
  			point2.abs.donneValeur(suiteAssociee.valeurs[i-1]);
  			point2.ord.donneValeur(suiteAssociee.valeurs[i]);
  			point1.positionne(false, dimf);
  			point2.positionne(false, dimf);
  			seg.positionne(false, dimf);
  			dist = seg.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  			point1.abs.donneValeur(suiteAssociee.valeurs[i-1]);
  			point1.ord.donneValeur(suiteAssociee.valeurs[i]);
  			point2.abs.donneValeur(suiteAssociee.valeurs[i]);
  			point2.ord.donneValeur(suiteAssociee.valeurs[i]);
  			point1.positionne(false, dimf);
  			point2.positionne(false, dimf);
  			seg.positionne(false, dimf);
  			dist = seg.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		}
  		if (traitsDeRappelSurAbscisses)
  		{
  			point1.abs.donneValeur(suiteAssociee.valeurs[i]);
  			point1.ord.donneValeur(0);
  			point2.abs.donneValeur(suiteAssociee.valeurs[i]);
  			point2.ord.donneValeur(suiteAssociee.valeurs[i]);
  			point1.positionne(false, dimf);
  			point2.positionne(false, dimf);
  			seg.positionne(false, dimf);
  			dist = seg.distancePoint(xp,yp,masquage);
  			if ((dist != -1) && (dist < CListeObjets.distanceMiniPourEtreProche)) return dist;
  		}
  	} // for  			
  	return -1;    
  }

  @Override
  public long getNature() {
    return NatureObjet.NGrapheSuiteRec;
  }  

  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(suiteAssociee) || suiteAssociee.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(repereAssocie) || repereAssocie.estDefiniParObjetDans(listeOb));
	}        
  

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    point1 = new CPointDansRepere();
    point2 = new CPointDansRepere();
    seg = new CSegment(list, point1, point2);
    int ind1 = inps.readCInt();
    int ind2 = inps.readCInt();
    suiteAssociee = (CSuiteRec) (list.get(ind1,CSuiteRec.class));
    repereAssocie = (CRepere) (list.get(ind2,CRepere.class));
    traitsDeRappelSurAbscisses = inps.readBoolean();
    point1.rep = repereAssocie;
    point2.rep = repereAssocie;   
   }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    point1 = new CPointDansRepere();
    point2 = new CPointDansRepere();
    seg = new CSegment(list, point1, point2);
    int ind1 = inps.readInt();
    int ind2 = inps.readInt();
    suiteAssociee = (CSuiteRec) (list.get(ind1,CSuiteRec.class));
    repereAssocie = (CRepere) (list.get(ind2,CRepere.class));
    traitsDeRappelSurAbscisses = inps.readBoolean();
    point1.rep = repereAssocie;
    point2.rep = repereAssocie;   
  }  

  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(suiteAssociee);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(repereAssocie);
    oups.writeInt(ind2);
    oups.writeBoolean(traitsDeRappelSurAbscisses);
  }     
}
