/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;
import java.io.*;
import java.awt.*;
import java.util.ListIterator;

import types.*;

import dialogs.LieuObjetDlg;
import dialogs.MtgDlg;
import entreesSorties.*;

import mathgraph32.MtgFrame;

/**
 *
 * @author Yves
 * Nouveau type d'objet pour la version 1.9.5
 */
public abstract class CLieuObjetAncetre extends CElementGraphique{
  public CElementGraphique elementAssocie; // l'objet graphique dont les positions gnrent les traces
  // Modification pour la versino 2.5. Le nombre de traces devient dynamique
  // protected int nombreTraces;
  public CValeurAssocieeAVariable nombreTraces;
  protected CSousListeObjets listeElementsAncetres; // Contiendra une liste de pointeurs vers les lments
  protected CListeObjets listeCopiesObjet; // Contiendra autant de copies de l'objet qu'il y a de traces
  protected static int nombreMaxiPourLieuObjet = 10000;
  
  
  /**
   * Creates a new instance of CLieuObjetAncetre
   */
  public CLieuObjetAncetre() {
    nombreTraces = new CValeurAssocieeAVariable();
  }
  
  // Constructeur pour clonage
  protected CLieuObjetAncetre(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit,
    CElementGraphique elementAssocieInit, CValeurAssocieeAVariable nombreTracesInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, 0d, 0d, masqueInit);
    elementAssocie = elementAssocieInit;
    nombreTraces = nombreTracesInit;
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Cette liste sera finie d'tre valide dans setClone()
    listeElementsAncetres = new CSousListeObjets();
    // idem pour la liste contenant les copies de l'objet associ servant  mmoriser
    // les positions de cet objet
    listeCopiesObjet = new CListeObjets();
  }
  
  @Override
  public void setClone(CElementBase ptel) {
    super.setClone(ptel);
    for (int i = 0; i < listeCopiesObjet.size(); i++) {
    	CElementBase elb1 = listeCopiesObjet.get(i);
    	CElementBase elb2 = ((CLieuObjetAncetre)ptel).listeCopiesObjet.get(i);
    	elb1.setClone(elb2);  	
    }
  }
  
  // Ajout pour version 3.0
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  	nombreTraces.positionne(infoRandom, dimfen);
  	existe = existe && nombreTraces.existe();
  }

  /**
   * Fonction n'existant pas en C++ et charge de mettre  jour l'objet quand un lment dont il dpend a t modifi.
   * Devra tre appel par les lieux de points et lieux d'objets pour remettre  jour les liste qu'ils utilisent
   * de faon interne
   */
  @Override
  public void metAJour() {
  	prepareListeCopiesObjet();
  	donneCouleur(couleur);
  }
  
  /**
   * Fonction rajoute pour la version 2.5.3 pour grer les lieux de lieux d'objets.
   * Renvoie 1 par dfaut et ne sera redfinie que pour les lieux d'objets
   * de faon que les figures ne soient pas trop lourdes
   * @return
   */
  @Override
  public int nombreObjetsPourLieuObjet()
	{
  	if (!nombreTraces.existe()) return Integer.MAX_VALUE;
		double d = Math.round(nombreTraces.rendValeur(false))* Math.round(elementAssocie.nombreObjetsPourLieuObjet());
		if ((d < 0) || (d > Integer.MAX_VALUE)) d = Integer.MAX_VALUE;
		return (int) d;
	}  
  
  @Override
  public String chaineDesignation() {
    return getString("designationLieuObjet");
  }

  /**
   * Prparation des listes dont le lieu d'objet a besoin
   * Doit tre appel immdiatement aprs cration de l'objet
   * mais aussi aprs modification de l'objet qui a gnr le lieu
   */
  public void prepareListeCopiesObjet() {
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
  	listeCopiesObjet.retireTout();
    double nbt = Math.round(nombreTraces.rendValeur(false));
    if ((nbt >=2) && (nbt <= CLieuObjetAncetre.nombreMaxiPourLieuObjet)) {
    	listeCopiesObjet.ajouteClonesDe(elementAssocie, (int)nbt);
      // Ligne suivante ajoute version 2.0
      // Ncessaire pour les lieux de lieux d'objets car ce type d'objet mainient une liste
      // qui est instancie de faon dynamique
      listeCopiesObjet.metAJour();
    }
    // La ligne suivante sert  donner aux objets clons servant  la trace la couleur du lieu
    // d'objet
    donneCouleur(couleur); 
  }
  
	/** A noter : dans le descendant CLieuObjetParPtLie qui est gnr par un point li
	 * il n'est pas utile de redfinir DependDe() car si le point li gnrateur
	 * dpend de P alors le pointATracer aussi puisqu'il dpend forcment
	 * du point li gnrateur (sinon la cration du lieu aurait t refuse)
	 */
  @Override
  public boolean dependDe(CElementBase p) {
    return super.dependDe(p) || elementAssocie.dependDe(p) || nombreTraces.dependDe(p);
  }
  
  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || elementAssocie.dependDePourReclassement(p) || nombreTraces.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || elementAssocie.dependDePourBoucle(p) || nombreTraces.dependDePourBoucle(p));
  }
  
  // Les deux fonctions suivantes pour que les lieux d'objets cachs apparaissent en pointills
  @Override
  public final void cache() {
    super.cache();
		ListIterator<CElementBase> li = listeCopiesObjet.listIterator();
    while (li.hasNext()) {
    	// affiche ne testant pas l'existence d'un lment, il faut tester avant affichage si l'lment clon existait
    	CElementGraphique elg = (CElementGraphique) li.next();
      elg.cache();
    }
  }

  @Override
  public final void montre() {
    super.montre();
		ListIterator<CElementBase> li = listeCopiesObjet.listIterator();
    while (li.hasNext()) {
    	// affiche ne testant pas l'existence d'un lment, il faut tester avant affichage si l'lment clon existait
    	CElementGraphique elg = (CElementGraphique) li.next();
      elg.montre();
    }
  }
  
  //  Il faut redfinir affiche pour les lieux d'objets affichant un affichage de valeur
  // qui a besoin d'appeler affiche pour pouvoir ensuite s'effacer correctement
  public void affiche(Graphics2D pDC, boolean masquage, Color couleurFond, float transparence) {
  	if (!(masquage && masque)) {
  		ListIterator<CElementBase> li = listeCopiesObjet.listIterator();
      while (li.hasNext()) {
      	// affiche ne testant pas l'existence d'un lment, il faut tester avant affichage si l'lment clon existait
      	CElementGraphique elg = (CElementGraphique) li.next();
        if (elg.existe) elg.affiche(pDC, masquage, couleurFond, transparence);
      }
    }
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence){
    ListIterator<CElementBase> li = listeCopiesObjet.listIterator();
    while (li.hasNext()) {
      ((CElementGraphique) li.next()).trace(pDC, couleurFond, transparence);
    }
  }
  
  @Override
  public long getNature() {
    return NatureObjet.NLieuObjet;
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
    if (!(existe) || (masquage && masque)) return -1; // Rectifi version 3.4.0.0 oubli avant 	
  	ListIterator<CElementBase> li = listeCopiesObjet.listIterator();
  	while (li.hasNext()) {
  		CElementGraphique ptel = (CElementGraphique) li.next();
  		if (ptel.existe) {
  			double d = ptel.distancePoint(xp, yp, masquage);
  			if ((d < CListeObjets.distanceMiniPourEtreProche) && (d != -1)) return d;
  		}
  	}
  	return -1;
  }
  
  @Override
  public boolean modifiableParMenu() {
    // Version 3.5 : les lieux d'objets qui sont des objets finaux restent modifiables
  	return true;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new LieuObjetDlg(frame, this, true);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil62");
  }  
  /* Fonction rajoute pour que les clones d'objest aient la mme couleur que le lieu d'objets */
  
  public void donneCouleur( Color coul) {
  	couleur = coul;
    ListIterator<CElementBase> li = listeCopiesObjet.listIterator();
    while (li.hasNext()) {
      ((CElementGraphique) li.next()).donneCouleur(coul);
    }
  }
  
  protected void ajusteListeCopieObjets() {
    int nbt = (int) Math.round(nombreTraces.rendValeur(false));
    int ancien = listeCopiesObjet.size();
    if (nbt < ancien) {
      for (int i = 0; i < ancien - nbt; i++) {
        listeCopiesObjet.remove(listeCopiesObjet.size()-1);
      }
    } else {
      listeCopiesObjet.ajouteClonesDe(elementAssocie, nbt - ancien);
      // Ligne suivante ajoute version 2.0
      // Ncessaire pour les lieux de lieux d'objets car ce type d'objet maintient une liste
      // qui est instancie de faon dynamique
      listeCopiesObjet.metAJour();
      donneCouleur(couleur);
    }
    // Pour les lieux de lieu d'objets crs avec ce lieu il faut remettre  jour
    // leur liste de copies d'objets
    listeProprietaire.recreeListesCopiesLieuxObjetsDependantDe(this);
  }
  
  /**
   * Retire tous les objets de la liste de copies d'objets et recre la liste
   */
  protected void recreeListeCopieObjets() {
    int nbt = (int) Math.round(nombreTraces.rendValeur(false));
    int size = listeCopiesObjet.size();
    for (int i = 0; i < size; i++) listeCopiesObjet.remove(listeCopiesObjet.size()-1);
    listeCopiesObjet.ajouteClonesDe(elementAssocie, nbt);
    listeCopiesObjet.metAJour();
    donneCouleur(couleur);
  }
  

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    elementAssocie = (CElementGraphique) (list.get(ind1,CElementGraphique.class));
    int nbt = inps.readCInt();
    nombreTraces.donneValeur(nbt);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    // Ligne suivante revue version 3.4.2
    // listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    listeElementsAncetres = new CSousListeObjets();
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      // idem pour la liste contenant les copies de l'objet associ servant  mmoriser
      // les positions de cet objet
      listeCopiesObjet = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
      // Comme le nombre d'objets est dynamique on ne peut pas savoir au chargement combien il y en a.
      //On laisse la liste vide et cela sera mis  jour lorsque positionne() sera appel
      donneCouleur(couleur);
    }  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return 
		(listeOb.contains(elementAssocie) || elementAssocie.estDefiniParObjetDans(listeOb))
		&&
		(nombreTraces.estDefiniParObjetDans(listeOb));
	}   
  
  // Chargement pour les versions antrieures depuis la version 2.5
  public void chargeDepuisFluxNew(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    elementAssocie = (CElementGraphique) (list.get(ind1,CElementGraphique.class));
    nombreTraces.chargeDepuisFlux(inps, list);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    // Ligne suivante revue version 3.4.2
    // listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    listeElementsAncetres = new CSousListeObjets();
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      // idem pour la liste contenant les copies de l'objet associ servant  mmoriser
      // les positions de cet objet
      listeCopiesObjet = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
      // Comme le nombre d'objets est dynamique on ne peut pas savoir au chargement combien il y en a.
      //On laisse la liste vide et cela sera mis  jour lorsque positionne() sera appel
    	// listeCopiesObjet.ajouteClonesDe(elementAssocie, 1);
      // La ligne suivante sert  donner aux objets clons servant  la trace la couleur du lieu
      // d'objet
      donneCouleur(couleur); 
    }
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    elementAssocie = (CElementGraphique) (list.get(ind1,CElementGraphique.class));
    nombreTraces.read(inps, list);
    // Il faut crer la liste des lments dont dpend le point  tracer
    // Pour cette liste, on fait pointer la longueur unit ventuelle sur la liste mre
    // Ligne suivante revue version 3.4.2
    // listeElementsAncetres = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
    listeElementsAncetres = new CSousListeObjets();    
    // Modification pour la version 2.5.
    // Certaines choses pas  faire pour le chargement d'un prototype
    if (list.getClass() != CPrototype.class) {
      // idem pour la liste contenant les copies de l'objet associ servant  mmoriser
      // les positions de cet objet
      listeCopiesObjet = new CListeObjets(list.uniteAngle, list.pointeurLongueurUnite);
      // Comme le nombre d'objets est dynamique on ne peut pas savoir au chargement combien il y en a.
      //On laisse la liste vide et cela sera mis  jour lorsque positionne() sera appel
      donneCouleur(couleur);
    }
  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(elementAssocie);
    oups.writeInt(ind1);
    nombreTraces.write(oups, list);
  }
  
  @Override
  public boolean estCapturableSouris() {
  	return elementAssocie.estDeNature(NatureObjet.NCommentaireOuLatex);
  }
  
}
