/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.*;

import java.awt.Color;

import types.*;

import dialogs.AbscisseDlg;
import dialogs.MtgDlg;
import entreesSorties.MathGraphDataInputStream;
import entreesSorties.MathGraphDataOutputStream;

import mathgraph32.MtgFrame;

public class CPointParAbscisse extends CPointAncetre {
	CPointAncetre origine;
	CPointAncetre extremite;
	public CValeurAssocieeAVariable abscisse;

  /** Creates new CPointParAbscisse */
  public CPointParAbscisse() {
    abscisse = new CValeurAssocieeAVariable();
  }

  // Constructeur pour clonage
  public CPointParAbscisse(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
      Color couleurInit, boolean nomMasqueInit, double dec_xInit, double dec_yInit,
      boolean masqueInit, String nomInit, int indiceTailleNomInit,
      int motifInit , CPointAncetre origineInit, CPointAncetre extremiteInit,
      CValeurAssocieeAVariable abscisseInit, boolean marquePourTraceInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, nomMasqueInit, dec_xInit, dec_yInit, 
        masqueInit, nomInit, indiceTailleNomInit, motifInit, marquePourTraceInit);
    origine = origineInit;
    extremite = extremiteInit;
    abscisse = abscisseInit;
  }
  
  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(origine);
    int ind2 = listeSource.indexOf(extremite);
    int ind3 = listeSource.indexOf(impProto);        
    CValeurAssocieeAVariable abscisseClone = (CValeurAssocieeAVariable) abscisse.getClone(listeSource, listeCible);
    CElementBase ptelb = new CPointParAbscisse(listeCible, (CImplementationProto)(listeCible.get(ind3, CImplementationProto.class)),
      estElementFinal, couleur, nomMasque, dec_x, dec_y, masque, nom, indiceTailleNom, motif,
      (CPointAncetre)(listeCible.get(ind1,CPointAncetre.class)), (CPointAncetre)(listeCible.get(ind2, CPointAncetre.class)),
      abscisseClone, marquePourTrace);
    return ptelb;
  }

  // Seulement pour l'applet il faut nommer les point servant  crer un point dfini par son
  // abscisse
  @Override
  public boolean nomIndispensable(CElementGraphique el) {
    return ((el == origine) || (el == extremite));
  }
  
  @Override
  public String infoPourHistorique() {
    return getString("chinfo16") + " "+ abscisse.chaineInfo() + "\n"
            + getString("chinfo32") + "(" + origine.rendNom()
            + "," + extremite.rendNom() + ")";
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(origine);
    liste.add(extremite);
  } 
  
  @Override
  public boolean dependDe(CElementBase p) {
  	if (elementTestePourDependDe == p) return dependDeElementTeste;
    return memoriseDependance(super.dependDe(p) || origine.dependDe(p) || extremite.dependDe(p) || abscisse.dependDe(p));
  }

  @Override
  public boolean dependDePourReclassement(CElementBase p) {
    return super.dependDePourReclassement(p) || origine.dependDePourReclassement(p)
    	|| extremite.dependDePourReclassement(p) || abscisse.dependDePourReclassement(p);
  }
  
  @Override
  public boolean dependDePourBoucle(CElementBase p) {
    return ((p == this) || origine.dependDePourBoucle(p) || extremite.dependDePourBoucle(p)
    || abscisse.dependDePourBoucle(p));
  }

  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	abscisse.positionne(infoRandom, dimfen);
    existe = abscisse.existe() && origine.existe && extremite.existe;
    if (!existe) return;
    VecteurDirecteur u = new  VecteurDirecteur(origine,extremite);
    double k1 = abscisse.rendValeur(infoRandom);
    VecteurDirecteur vec = new VecteurDirecteur();
    vec.x = k1*u.x;
    vec.y = k1*u.y;
    x = origine.x + vec.x;
    y = origine.y + vec.y;
    super.positionne(infoRandom, dimfen);
  }
    

  
  @Override
  public boolean confonduAvec(CElementBase p) {
    CPointParAbscisse pt;
    if (p.getClass() == this.getClass()) {
      pt = (CPointParAbscisse) p;
      return (origine == pt.origine) && (extremite == pt.extremite)
        && abscisse.confonduAvec(pt.abscisse);
    }
    else return false;
  }        

  @Override
  public void remplacePoint(CPointAncetre ancienPoint, CPointAncetre nouveauPoint) {
    if (origine == ancienPoint) origine = nouveauPoint;
    if (extremite == ancienPoint) extremite = nouveauPoint;
  }
  
  @Override
	public boolean estDefiniParObjetDans(CListeObjets listeOb) {
  	return
		(listeOb.contains(origine) || origine.estDefiniParObjetDans(listeOb))
		&&
		(listeOb.contains(extremite) || extremite.estDefiniParObjetDans(listeOb))
		&&
		abscisse.estDefiniParObjetDans(listeOb);
	}        
  
  @Override
  public boolean modifiableParMenu() {
  	// Modification version 3.5
  	// return true;
  	return !estElementFinal;
  }
  
   @Override
  public MtgDlg dialogueModification(MtgFrame frame) {
  	return new AbscisseDlg(frame, this, true);
  }
  
  @Override
  public String informationModification() {
  	return getString("chaineOutil52");
  }  

  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    int ind1 = inps.readCInt();
    origine = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    int ind2 = inps.readCInt();
    extremite = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    abscisse.chargeDepuisFlux(inps, list);
  }

  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    int ind1 = inps.readInt();
    origine = (CPointAncetre) (list.get(ind1,CPointAncetre.class));
    int ind2 = inps.readInt();
    extremite = (CPointAncetre) (list.get(ind2,CPointAncetre.class));
    abscisse.read(inps, list);  }
  
  @Override
  public void write(MathGraphDataOutputStream oups, CListeObjets list)  throws IOException {
    super.write(oups, list);
    int ind1 = list.indexOf(origine);
    oups.writeInt(ind1);
    int ind2 = list.indexOf(extremite);
    oups.writeInt(ind2);
    abscisse.write(oups, list);
  }
  
  @Override
  public boolean appartientParDefinition(CDroiteAncetre droite)
  {
    return super.appartientParDefinition(droite) || (origine.appartientParDefinition(droite)) && (extremite.appartientParDefinition(droite));
  }
  
}
