/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.awt.*;
import java.io.IOException;
import entreesSorties.MathGraphDataInputStream;
import types.*;
import mathgraph32.MtgFrame;

/** Classe reprsentant une surface dlimite par un cercle
 * Modifie version 3.0 pour pouvoir tre un objet gnrant un lieu d'objets
 * @author Yves
 *
 */
public class CSurfaceDisque extends CSurface {
	private CCercle cercle; // Ajout version 2.0
  /** Creates new CSurfaceDisque */
  public CSurfaceDisque() {
  }
  
  // Constructeur pour clonage
  public CSurfaceDisque(CListeObjets liste, CImplementationProto impProtoInit, boolean estElementFinalInit,
    Color couleurInit, boolean masqueInit, int styleRemplissageInit, CCercle bordInit) {
    super(liste, impProtoInit, estElementFinalInit, couleurInit, masqueInit, styleRemplissageInit, bordInit);
    cercle = new CCercle(liste, null, false, Color.black, false, StyleTrait.traitFin);
  }

  @Override
  public CElementBase getClone(CListeObjets listeSource, CListeObjets listeCible) {
    int ind1 = listeSource.indexOf(bord);
    int ind2 = listeSource.indexOf(impProto);        
    CElementBase ptelb = new CSurfaceDisque(listeCible, (CImplementationProto)(listeCible.get(ind2, CImplementationProto.class)),
      estElementFinal, couleur, masque, styleRemplissage, (CCercle)(listeCible.get(ind1, CCercle.class)));
    return ptelb;
  }

  @Override
  public String infoPourHistorique() {
    return getString("chinfo61");
  }

  @Override
  public void ajouteAntecedents(MtgFrame frame, CListeObjets liste) {
    liste.add(bord);
  }
  
  @Override
  public double distancePoint(double xp, double yp, boolean masquage) {
  	if (!existe || (masquage && masque)) return -1;
  	return cercle.distancePointPourSurface(xp, yp, masquage);
  }
  
  // Rajout version 3.0 pour pouvoir tre un objet gnrant un lieu d'objets
  @Override
  public void positionne(boolean infoRandom, DimensionFenetre dimfen) {
  	super.positionne(infoRandom, dimfen);
  	cercle.setClone((CCercle) bord);
  }
  
  // Ajout version 3.0
  @Override
	public void setClone(CElementBase ptel) {
		super.setClone(ptel);
		cercle.setClone(((CSurfaceDisque) ptel).cercle);
  }
  
  @Override
  public void trace(Graphics2D pDC, Color couleurFond, float transparence) {

    int styleRemplissageEffectif;
    if (masque) styleRemplissageEffectif = StyleRemplissage.QuadrillageCroise;
    else styleRemplissageEffectif = styleRemplissage;
    if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePleinTransp) {
    	if (transparence != 1) {
    		pDC.setColor(couleur);
  	    Composite originalComposite = pDC.getComposite();
  	    pDC.setComposite(AlphaComposite.getInstance(AlphaComposite.SRC_OVER, transparence));
  	    // Ligne suivante modifie version 2.0
  	    pDC.fillOval((int)Math.round(cercle.xCarre1), (int)Math.round(cercle.yCarre1),
  	      (int)Math.round(cercle.xCarre2 - cercle.xCarre1 + 1), (int)Math.round(cercle.yCarre2 - cercle.yCarre1 + 1));
  	    pDC.setComposite(originalComposite);    		
    	}
    	else {
    		pDC.setColor(couleur);
  	    // Ligne suivante modifie version 2.0
  	    pDC.fillOval((int)Math.round(cercle.xCarre1), (int)Math.round(cercle.yCarre1),
    	      (int)Math.round(cercle.xCarre2 - cercle.xCarre1 + 1), (int)Math.round(cercle.yCarre2 - cercle.yCarre1) + 1);
    	}
    }
    else {
      if (styleRemplissageEffectif == StyleRemplissage.QuadrillagePlein) {
    		pDC.setColor(couleur);
  	    pDC.fillOval((int)Math.round(cercle.xCarre1), (int)Math.round(cercle.yCarre1),
    	      (int)Math.round(cercle.xCarre2 - cercle.xCarre1 + 1), (int)Math.round(cercle.yCarre2 - cercle.yCarre1) + 1);
      }
      else {
	    	TexturePaint tp = StyleRemplissage.getTexturePaint(couleur, styleRemplissageEffectif, listeProprietaire, couleurFond);
	    	pDC.setPaint(tp);
		    // Ligne suivante modifie version 2.0
	    	pDC.fillOval((int)Math.round(cercle.xCarre1), (int)Math.round(cercle.yCarre1),
		      (int)Math.round(cercle.xCarre2 - cercle.xCarre1 + 1), (int)Math.round(cercle.yCarre2 - cercle.yCarre1 + 1));
      }
    }
  }
  
  @Override
  public boolean peutGenererLieuObjet() {
  	return true;
  }
  
  // Ajout version 3.0 pour initialiser cercle
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException { 	
    super.chargeDepuisFlux(inps, list);
    cercle = new CCercle(list, null, false, Color.black, false, StyleTrait.traitFin);
  }

  
  // Ajout version 3.0 pour initialiser cercle
  @Override
  public void read(MathGraphDataInputStream inps, CListeObjets list)  throws IOException {
    super.read(inps, list);
    cercle = new CCercle(list, null, false, Color.black, false, StyleTrait.traitFin);
  }
}
