/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package objets;

import java.io.IOException;

import entreesSorties.MathGraphDataInputStream;


public class CValeurAssocieeAVariable extends CValeurAssocieeAVariableAncetre {
  // public String chaineCalculValeur;
  
  /** Creates new CValeurAssocieeAVariable */
  public CValeurAssocieeAVariable() {
  }
  
  public CValeurAssocieeAVariable(CListeObjets list) {
  	super(list);
  }

  /** Constructeur pour clonage */
  public CValeurAssocieeAVariable(CListeObjets liste, CCalculBase calculInit, double valeurInit) {
    super(liste, calculInit);
    valeur = valeurInit;
  }
  
  // Constructeur pour une valeur qui vient d'tre cre mais n'a pas t encore calcule
  public CValeurAssocieeAVariable(CListeObjets liste, CCalculBase calculInit) {
    super(liste);
    calcul = calculInit;
  }
  
  /** Constructeur  partir d'un autre objet du mme type */
  public CValeurAssocieeAVariable(CValeurAssocieeAVariable valInit) {
  	super(valInit.listeProprietaire);
    calcul = valInit.calcul.getClone(listeProprietaire, listeProprietaire);
    estConstant = valInit.estConstant;
    if (estConstant) valeur = valInit.valeur;
    existe = valInit.existe;
  }
  
  /** Constructeur  partir d'une valeur numrique */
  public CValeurAssocieeAVariable(CListeObjets liste, double valInit) {
  	super(liste);
  	calcul = new CConstante(liste,valInit);
  	valeur = valInit;
  	dejaPositionne = true;
    estConstant = true;
    existe = true;
  }
  
  public CValeurAssocieeAVariable getClone(CListeObjets listeSource, CListeObjets listeCible) {
    CCalculBase calculClone = calcul.getClone(listeSource, listeCible);
    /* Modifi version 4.4
    return new CValeurAssocieeAVariable(listeCible, calculClone, valeur);
    */
    CValeurAssocieeAVariable va = new CValeurAssocieeAVariable(listeCible, calculClone, valeur);
    va.dejaPositionne = dejaPositionne;
    va.estConstant = estConstant;
    va.valeur = valeur;
    va.existe = existe;
    return va;
  }
    
  @Override
  public void chargeDepuisFlux(MathGraphDataInputStream inps, CListeObjets list) throws IOException {
    super.chargeDepuisFlux(inps, list);
    // On lit la chane reprsentant le calcul en C++ qui n'est plus utilise
    @SuppressWarnings("unused")
		String s = inps.readCString();
  }
}
