/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import objets.CCentreGravite;
import objets.CListeObjets;
import objets.CMilieu;
import objets.CSegment;
import types.Couleur;
import types.MotifPoint;
import mathgraph32.MtgFigurePanel;
import mathgraph32.MtgFrame;

public class OutilCentreGravite extends OutilPar3pts {
  public OutilCentreGravite(MtgFrame frameInit) {
    super(frameInit);
  }

  @Override
  public void creeObjet() {
    CCentreGravite pt = new CCentreGravite(frame.cadreActif().listePr, null, false, 
    		frame.couleurActive(), false, 0d, 0d, false, frame.genereNomPourPoint(), frame.pref_indiceTaillePoliceNom, frame.stylePointActif,
    		point1, point2, point3, false);
    if (frame.ajouteElementAvecVerification(pt))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil51"));
    annuleClignotement();
    // Provisoire
    select();
  }

  @Override
  public void ajouteObjetsVisuels() {
    CListeObjets liste = frame.listeObjetsVisuels;
    MtgFigurePanel pane = frame.cadreActif().paneFigure;
  	CSegment pts1 = new CSegment(liste, null, false, point1, point2);
  	liste.add(pts1);
  	CSegment pts2 = new CSegment(liste, null, false, point2, pane.pointSouris);
  	liste.add(pts2);
  	CSegment pts3 = new CSegment(liste, null, false, pane.pointSouris, point1);
  	liste.add(pts3);
  	// On cre trois milieux
  	CMilieu ptm1 = new CMilieu(liste, null, false, Couleur.TraduitCouleur(1), MotifPoint.petitRond,
        point1, point2);
  	ptm1.cache();
  	liste.add(ptm1);
  	CMilieu ptm2 = new CMilieu(liste, null, false, Couleur.TraduitCouleur(1), MotifPoint.petitRond,
        point2, pane.pointSouris);
  	ptm2.cache();
  	liste.add(ptm2);
  	CMilieu ptm3 = new CMilieu(liste, null, false, Couleur.TraduitCouleur(1), MotifPoint.petitRond,
        point1, pane.pointSouris);
  	ptm3.cache();
  	liste.add(ptm3);

  	// On cre trois segments mdianes
  	CSegment ptmed1 = new CSegment(liste, null, false, point1, ptm2);
  	liste.add(ptmed1);
  	CSegment ptmed2 = new CSegment(liste, null, false, point2, ptm3);
  	liste.add(ptmed2);
  	CSegment ptmed3 = new CSegment(liste, null, false, pane.pointSouris, ptm1);
  	liste.add(ptmed3);
  	
  	// On cre un centre de gravit provisoire
  	CCentreGravite ptcg = new CCentreGravite(frame.cadreActif().listePr, null, false, frame.couleurActive(), false, 0d, 0d,
  			false, "", 2, frame.stylePointActif, point1, point2, pane.pointSouris, false);
  	liste.add(ptcg);
  }  
  @Override
  public String chaineIndication (int indEtape) {
    switch (indEtape) {
      case 1 : return getString("centreGravite1");
      case 2 : return getString("centreGravite2");
      case 3 : return getString("centreGravite3");
      default: return "";
    }
  }	
}
