/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Point;
import java.util.ListIterator;

import objets.CCalcul;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CValeurDynamique;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import types.NatureCalcul;
import types.NatureObjet;

public class OutilConstructionChoixFinauxGraphiques extends Outil {
  public OutilConstructionChoixFinauxGraphiques(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public void select() {
    super.select();
    MtgInternalFrame cadre = frame.cadreActif();
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageActif.designationDe(NatureObjet.NToutObjet);
  	// Pour qu'on ne puisse dsigner que des objets construits avec les sources	
		CListeObjets liste = cadre.listePr;
		CListeObjets listo = new CListeObjets();
		listo.ajouteElementsDe(cadre.listeObjetsSourcesG);
		listo.ajouteElementsDe(cadre.listeObjetsSourcesNG);
		// Modifi version 3.4.0
		// On rajoute  listo les variables telles qu'il existe des objets dpendant
		//  la fois des objets sources et de la variablealcul et dpendant d'au moins un objet source
		// ou (version 3.5.0) les calculs rels ou complexes constants tels qu'il existe au moins
		// un objet dpendant de ce c
  	ListIterator<CElementBase> li = liste.listIterator();
  	int i = 0;
    while (li.hasNext()) {
    	CElementBase elb = li.next();
    	if (elb.estDeNatureCalcul(NatureCalcul.NVariable)) {
    		if (!listo.contains(elb)) {
    			CValeurDynamique val = (CValeurDynamique) elb;
    			for (int j = i; j < liste.size(); j++) {
    				CElementBase elb2 = liste.get(j);
    				if (elb2.dependDe(val) && elb2.dependDeAuMoinsUn(listo)) listo.add(val);
    			}
    		}
    	}
    	else {
      	if (elb.estDeNatureCalcul(NatureCalcul.NToutCalculOuFonctionReelleOuComplexe)) {
      		if (!listo.contains(elb)) {
      			CCalcul val = (CCalcul) elb;
      			if (val.estConstant()) {
	      			for (int j = i; j < liste.size(); j++) {
	      				CElementBase elb2 = liste.get(j);
	      				if (elb2.dependDe(val) && elb2.dependDeAuMoinsUn(listo)) listo.add(val);
	      			}
      			}
      		}
      	}    		
    	}
    	i++;
    }
    
  	// On donne la possibilit de cliquer sur un objet source seulement
  	// dans un cas : si c'est un point libre qui figure parmi les objets sources
  	// Attention : il faut permettre de dsigner un point libre quand il figure dans
  	// les objets sources mais pas un point libre  coorodnnes entires
  	// d'o ne pas utiliser pta->Nature qui renvoie NPointBase pour les deux
  	// mais utilisation de GetRuntimeClass()
    li = liste.listIterator();
    while (li.hasNext()) {
    	CElementBase elb = li.next();
    	if (elb.getNature() == NatureObjet.NPointBase) {
    		if (!cadre.listeObjetsSourcesG.contains(elb)) frame.listeExclusionDesignation.add(elb);
    	}
    	else {
    		if (!elb.estDefiniParObjetDans(listo)) frame.listeExclusionDesignation.add(elb);
    	}
    }
    
    frame.listeClignotante.ajouteElementsDe(cadre.listeObjetsFinauxG);
    frame.listeExclusionDesignation.ajouteElementsDe(cadre.listeObjetsFinauxG);
    frame.indication(getString("objetsFinauxGraphiques1"));
    resetClignotement();
	}
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
		if (elg.getNature() == NatureObjet.NPointBase) {
  		if (!frame.dialogueConfirmation_YES_NO(getString("ch150"))) return;
  	}
  	frame.listeClignotante.add(elg);
  	frame.listeExclusionDesignation.add(elg);
  	frame.cadreActif().listeObjetsFinauxG.add(elg);
  }
  
  @Override
  public void creeObjetApresClicDroit() {
  	frame.activeOutilCapture();
  }
	
}
