/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Cursor;
import java.awt.Point;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CElementGraphique;
import objets.CLatex;
import objets.CListeObjets;
import types.StyleEncadrement;
import dialogs.LatexDlg;

public class OutilLatex extends Outil {
  Point pointDesigne;
  
  /** Creates a new instance of OutilLatex */
  public OutilLatex(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public Cursor curseurUtilise() {
		return MtgFrame.cornerCursor;
	}

  @Override
	public void select() {
    super.select();
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("commentaire1"));
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
    pointDesigne = pointClique;
    creeObjet();
  }
  
  @Override
  public void creeObjet() {
  	MtgInternalFrame cadre = frame.cadreActif();
    int x = pointDesigne.x;
    int y = pointDesigne.y;
    CListeObjets liste = cadre.listePr;
    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
    // pour matrialiser l'emplacement de l'affichage
    CLatex latexClignotant = new CLatex(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
    		StyleEncadrement.Sans, false, cadre.couleurFond, "\\text{" + getString("ch80") + "}",
				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
    latexClignotant.positionne(false, cadre.dimf);
    ajouteClignotementDe(latexClignotant);
    resetClignotement();
    CLatex latex = new CLatex(liste, null, false, frame.couleurActive(),
    		x, y, 0d, 0d, false, null, frame.pref_IndiceTaillePoliceAffichages, StyleEncadrement.Sans, false, cadre.couleurFondAffichage, null,
				CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
    
    frame.indication("");
    LatexDlg dlg = new LatexDlg(frame, latex, false, latexClignotant);
		dlg.setLocationRelativeTo(frame);
    dlg.setVisible(true);
    if (dlg.retourOK()) {
	    if (frame.ajouteElementAvecVerification(latex)) {
	    	annuleClignotement();
	      cadre.gestionnaire.enregistreFigureEnCours(getString("chaineOutil140"));
	      cadre.repaintFigureAvecCalcul();
	    }
    }
    annuleClignotement();
    select();
  }
}
