/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import java.awt.Cursor;
import java.awt.Point;
import java.util.ListIterator;
import dialogs.MacroAffectationPointLieDlg;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CAffichageLieAPoint;
import objets.CCommentaire;
import objets.CElementBase;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CMacroAffectationPointLie;
import objets.CPointLie;
import types.NatureObjet;
import types.StyleEncadrement;

public class OutilMacroAffectationPointLie extends Outil {
	private CPointLie pointLie1;
	private CPointLie pointLie2;
	private CMacroAffectationPointLie macro;
	private boolean dialogueRempli;
	
  /** Creates a new instance of OutilPtLib */
  public OutilMacroAffectationPointLie(MtgFrame frameInit) {
    super(frameInit, true);
  }
    
  @Override
	public Cursor curseurUtilise() {
		if (dialogueRempli) return MtgFrame.crossHairCursor;
		else return MtgFrame.cornerCursor;
	}

  @Override
	public void select() {
    super.select();
    frame.outilPointageActif = frame.outilPointageClic;
    frame.indication(getString("macro1"));
    dialogueRempli = false;
    pointLie1 = null;
    pointLie2 = null;
  }
	
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	CCommentaire comClignotant = null;
  	int x = pointClique.x;
    int y = pointClique.y;
    MtgInternalFrame cadre = frame.cadreActif();
    CListeObjets liste = cadre.listePr;
    if (!dialogueRempli) {
	    // Avant l'affichage du dialogue on cre un message qui sera ajout  la liste des objets clignotants
	    // pour matrialiser l'emplacement de l'affichage
	    comClignotant = new CCommentaire(liste, null, false, frame.couleurActive(), x, y, 0d, 0d, false, null, 2,
	    		StyleEncadrement.Sans, false, frame.cadreActif().couleurFond, getString("ch80"),
					CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    comClignotant.positionne(false, frame.cadreActif().dimf);
	    ajouteClignotementDe(comClignotant);
	    frame.listeExclusionDesignation.add(comClignotant);
	    resetClignotement();
	    macro = new CMacroAffectationPointLie(liste, null, false, frame.couleurActive(), 
	    		x, y, 0d, 0d, false, null, 2, false, cadre.couleurFondAffichage, "", "", null, null,
	    		CAffichageLieAPoint.alignementHorizontalGauche, CAffichageLieAPoint.alignementVerticalHaut);
	    frame.indication("");
	    MacroAffectationPointLieDlg dlg = new MacroAffectationPointLieDlg(frame, macro, false);
			dlg.setLocationRelativeTo(frame);
	    dlg.setVisible(true);
	    if (dlg.retourOK()) {
	    	frame.indication(getString("macroAffectationPointLie1"));				
	    	dialogueRempli = true;
	    	frame.listeClignotante.ajouteObjetsParNature(liste,NatureObjet.NPointLie);
	    	frame.outilPointageActif = frame.outilPointageObjetClignotant;
	    	frame.outilPointageActif.designationDe(NatureObjet.NPointLie);
	    	frame.outilPointageActif.reset();
	    }
	    else {
	    	annuleClignotement();
	    	frame.activeOutilCapture();
	    }
    }
    else {
    	if (pointLie1 == null) {
    		// S'il n'y a pas d'autre point li au mme objet qui soit visible, abandon
    		pointLie1 = (CPointLie) elg;
    		if (liste.nombrePointsLiesAVisibles(pointLie1) == 1) {
    			frame.dialogueAvertissement(getString("ch130"));
    			frame.activeOutilCapture();
    			return;
    		}
    		annuleClignotement();
    		// On ne garde comme objet clignotant que le commentaire clignotant
    		for (int i = 0; i < frame.listeClignotante.size() - 1; i++) frame.listeClignotante.remove(1);
	    	frame.listeClignotante.add(pointLie1.lieA());
	    	frame.listeExclusionDesignation.add(pointLie1);
	    	resetClignotement();
	    	// On ajoute  la liste clignotante les points lis au mme objet que le point li qui a t dsign
	    	ListIterator<CElementBase> li = liste.listIterator();
	    	while (li.hasNext()) {
	    		CElementBase elb = li.next();
	    		if (elb.getNature() == NatureObjet.NPointLie) {
	    			CPointLie ptl = (CPointLie) elb;
	    			if (ptl.existe && (ptl != pointLie1) && (ptl.lieA() == pointLie1.lieA())) {
	    				frame.listeClignotante.add(ptl);
	    			}
	    		}
	    	}
	    	frame.indication(getString("macroAffectationPointLie2"));				
	    	frame.outilPointageActif.reset();   	
    	}
    	else {
    		pointLie2 = (CPointLie) elg;
    		macro.pointLieAPositionner = pointLie1;
    		macro.pointLieAssocie = pointLie2;
      	annuleClignotement();
  			if (frame.ajouteElementAvecVerification(macro)) {
  			  frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil103"));
  			  frame.cadreActif().repaintFigureAvecCalcul();
  			}
  			frame.activeOutilCapture();
    	}
    }
  }  
}
