/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/
package outils;

import java.awt.Cursor;
import java.awt.Point;
import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCercle;
import objets.CElementGraphique;
import objets.CListeObjets;
import objets.CPointInterieurCercle;
import objets.CPointInterieurPolygone;
import objets.CPolygone;
import objets.CSurfaceDisque;
import objets.CSurfacePolygone;
import types.Bool;
import types.Couleur;
import types.Doub;
import types.Int;
import types.NatureObjet;
import types.StyleRemplissage;

public class OutilPointInterieur extends Outil {

	private CElementGraphique objet1;
	private CPointInterieurPolygone pointInterieurPolygone;
	private CPointInterieurCercle pointInterieurCercle;
	
  /** Creates a new instance of OutilPointParMultVec */
  public OutilPointInterieur(MtgFrame frameInit) {
    super(frameInit, true);
  }
	
  @Override
	public Cursor curseurUtilise() {
		return MtgFrame.crossHairCursor;
	}
  
  @Override
  public void select() {
    super.select();
    objet1 = null;
    pointInterieurPolygone = null;
    pointInterieurCercle = null;
    frame.indication(getString("pointInterieur1"));
    frame.outilPointageCre.designationDe(NatureObjet.NPolygone | NatureObjet.NCercle);
    frame.outilPointageActif = frame.outilPointageCre;
    frame.outilPointageCre.reset();
  }
  
  @Override
  public void traiteObjetDesigne(CElementGraphique elg, Point pointClique) {
  	MtgInternalFrame cadre = frame.cadreActif();
  	CListeObjets liste = cadre.listePr;
    if (objet1 == null) {
      objet1 = elg;
      ajouteClignotementDe(elg);
      frame.outilPointageActif = frame.outilPointageClic;
      frame.indication(getString("pointInterieur2"));
      resetClignotement();
      cadre.videDesignation();
      ajouteObjetsVisuels();
    }
    else {
      int x = pointClique.x;
      int y = pointClique.y;
      boolean bpointInt = false;
      if (objet1.getNature() == NatureObjet.NPolygone) {
      	CPolygone poly = (CPolygone) objet1;
	      // On regarde si le point cliqu est bien intrieur au polygone
	      if (poly.pointInterieur(x, y)) {
	      	Bool estSurCote = new Bool(false);
	      	Int indicePremierSommet = new Int();
	      	Doub alpha = new Doub();
	      	Doub beta = new Doub();
	      	Doub gamma = new Doub();
	      	bpointInt = poly.determinePositionInterieure(x, y, indicePremierSommet, estSurCote, alpha, beta, gamma);
	      	if (bpointInt) {
	      		pointInterieurPolygone = new CPointInterieurPolygone(liste, null, false, frame.couleurActive(),
	      			false, 0d, 0d, false, "", frame.pref_indiceTaillePoliceNom, frame.stylePointActif, true, poly,
	      			estSurCote.getValue(), indicePremierSommet.getValue(),
	      			alpha.getValue(), beta.getValue(), gamma.getValue(), false);
	      		pointInterieurPolygone.positionne(false, cadre.dimf);
		      	if (pointInterieurPolygone.existe) creeObjet();
		      	else bpointInt = false;
		      }
	      }
	      if (!bpointInt) frame.dialogueAvertissement(getString("ch169"));
	      frame.activeOutilCapture();
	    }
      else {
      	CCercle cercle = (CCercle) objet1;
	      // On regarde si le point cliqu est bien intrieur au polygone
      	bpointInt = cercle.pointInterieur(x, y);
	      if (bpointInt) {
      		pointInterieurCercle = new CPointInterieurCercle(liste, null, false, frame.couleurActive(),
      			false, 0d, 0d, false, "", frame.pref_indiceTaillePoliceNom, frame.stylePointActif,
      			true, cercle, 0, 0, false);
      		pointInterieurCercle.placeEn(x, y);
      		pointInterieurCercle.positionne(false, cadre.dimf);
	      	if (pointInterieurCercle.existe) creeObjet();
	      	else bpointInt = false;
	      }
	      if (!bpointInt) frame.dialogueAvertissement(getString("ch169"));
	      frame.activeOutilCapture();      	
      }
    }
  }
  
@Override
public void ajouteObjetsVisuels() {
  CListeObjets liste = frame.listeObjetsVisuels;
	 if (objet1.getNature() == NatureObjet.NPolygone) {
		CSurfacePolygone surf = new CSurfacePolygone(liste, null, false, Couleur.TraduitCouleur(7), false,
				StyleRemplissage.QuadrillagePleinTransp, (CPolygone) objet1);
		frame.ajouteObjetVisuel(surf);
	}
	 else {
		CSurfaceDisque surf = new CSurfaceDisque(liste, null, false, Couleur.TraduitCouleur(7), false,
				StyleRemplissage.QuadrillagePleinTransp, (CCercle) objet1);
		frame.ajouteObjetVisuel(surf);
	 }
}  
  
  @Override
  public void creeObjet() {
  	CElementGraphique elg = (pointInterieurCercle == null) ? pointInterieurPolygone : pointInterieurCercle;
  	elg.donneNom(frame.genereNomPourPoint());
  	if (frame.ajouteElementAvecVerification(elg))
      frame.cadreActif().gestionnaire.enregistreFigureEnCours(getString("chaineOutil139"));
    annuleClignotement();
    select();
  }
  
  @Override
	public boolean activationValide() {
		return frame.cadreActif().listePr.nombreObjetsParNatureVisibles(NatureObjet.NPolygone | NatureObjet.NCercle)>0;
	}
	
	@Override
	public String messageActivationNonValide() {
		return getString("messagePasDePolygoneOuCercle");
	}	
  
  
}
