/*
MathGraph32 Java. Software for creating and animating dynamic mathematics figures
    Copyright (C) 2009  Yves Biton (France) http://mathgraph32.org 

    This program is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program.  If not, see <http://www.gnu.org/licenses/>.
*/

package outils;

import glob.Kernel;

import java.awt.event.MouseEvent;
import java.awt.geom.Point2D;


import mathgraph32.MtgFrame;
import mathgraph32.MtgInternalFrame;
import objets.CCercle;
import objets.CDroiteAncetre;
import objets.CElementGraphique;
import objets.CListeObjets;
import types.InfoProx;
import types.NatureObjet;
import types.PositionCercleCercle;
import types.PositionDroiteCercle;
import types.PositionDroites;
import dialogs.ChoixTypeDlg;
import dialogs.PremierDernierDlg;

public class OutilPointageIntersection extends OutilPointage {
  /** Creates a new instance of OutilPointageIntersection */
  public OutilPointageIntersection(MtgFrame frameInit) {
    super(frameInit);
  }
  
  /**
   * Renvoie la liste  utiliser.
   * Devra tre redfini pour l'outil de suppresion d'objet
   * qui ne permettra de supprimer que les objets de la liste
   * listeConstruction
   */
  protected CListeObjets listeUtilisee() {
    return frame.cadreActif().listePr;
  }
  
  
  public void mouseMoved(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
    int x = mouseEvent.getX();
    int y = mouseEvent.getY();
    CListeObjets liste = listeUtilisee();
    InfoProx info = frame.infoProx;
    int nbObjetsProches = liste.procheDe(aDesigner, mouseEvent.getPoint(), info, 
      frame.listeExclusionDesignation, true);
    int nbTypesProches = info.nombreTypesProches;
    boolean designationDejaAffichee = (cadre.commentaireDesignation().chaineCommentaire.length() != 0);
      // true si une dsignation est dj affiche prs du pointeur
    if (nbObjetsProches > 0) {
      String ch = "";
      CElementGraphique premierVoisin = info.premierVoisin();
      CElementGraphique dernierVoisin = info.dernierVoisin();
      // Si un premier objet n'a pas dj t cliqu et si deux objets sont proches du pointeur souris
      // et s'ils sont scants on peut crer l'intersection d'un seul clic
      if ( (frame.listeExclusionDesignation.size() == 0) && (nbObjetsProches == 2)
      		&& objetsSecants(premierVoisin, dernierVoisin))  ch = getString("designationIntersection");
      else {
	      // Si lors d'un dialogue de choix d'objet proche ona dj choisi un objet
	      // et si celui-ci est proche du pointeur souris, c'est celui-l qui sera slectionnn
	      if (info.premierPointVoisin != null) {
	        ch = getString("designationPoint");
	      }
	      else {
	        if (nbTypesProches == 1) {
	          for (int i =0; i < NatureObjet.nombreTypesDesignables; i++) {
	            if (info.infoParType[i].nombreVoisins > 0) {
	              ch = info.infoParType[i].premierVoisin.chaineDesignation();
	              break;
	            }
	          }
	        }
	        else {
	          ch = getString("designationAPreciser");
	        }
	      }
      }
      cadre.commentaireDesignation().donneChaine(ch);
      cadre.commentaireDesignation().placeEn(x, y);
      cadre.paneFigure.repaint();
    }
    else {
      setActiveCursor(MtgFrame.crossHairCursor);
      frame.cadreActif().videDesignation();
      if ((frame.listeObjetsVisuels.size() > 0) || designationDejaAffichee) {
        cadre.paneFigure.repaint();
      }
    }
  }
  public void mousePressed(java.awt.event.MouseEvent mouseEvent, MtgInternalFrame cadre) {
  	if (mouseEvent.getButton() == MouseEvent.BUTTON1) {
	    InfoProx info = frame.infoProx;
	    int nbObjetsProches = info.nombreObjetsProches;
	    if (info.nombreObjetsProches != 0) {
	      CElementGraphique premierVoisin = info.premierVoisin();
	      CElementGraphique dernierVoisin = info.dernierVoisin();
	      // Si un premier objet n'a pas dj t cliqu et si deux objets sont proches du pointeur souris
	      // et s'ils sont scants on peut crer l'intersection d'un seul clic
	      if ( (frame.listeExclusionDesignation.size() == 0) && (nbObjetsProches == 2)
	      		&& objetsSecants(premierVoisin, dernierVoisin)) {
	      	frame.outilInt.traiteObjetsDesignes(info.premierVoisin(), info.dernierVoisin(), mouseEvent.getPoint());
	      }
	      else {
	      	CElementGraphique elg;
		      if (info.nombreTypesProches == 1) {
		        int i;
		        for (i =0; i < NatureObjet.nombreTypesDesignables; i++) {
		          if (info.infoParType[i].nombreVoisins > 0) break;
		        }
		        // i contient l'indice du type d'objets proches
		        if (info.infoParType[i].nombreVoisins >= 2) {
		          // On ouvre un dialogue pour demander de choisir entre le premier et le dernier objet
		          PremierDernierDlg dlg = new PremierDernierDlg(frame);
		      		dlg.setLocationRelativeTo(frame);
		          dlg.setVisible(true);
		          if (!dlg.retourOK()) return;
		          // On regarde quelle a t la rponse de l'utilisateur
		          if (dlg.getPremierChoisi()) elg = info.infoParType[i].premierVoisin;
		          else elg = info.infoParType[i].dernierVoisin;
		        }
		        else {
		          elg = info.infoParType[i].premierVoisin;
		        }
		      }
		      else {
		        // Il faut d'abord ouvrir un dialogue demandant le type de l'objet
		        // que l'utilisateur souhaite dsigner
		        ChoixTypeDlg dlgct = new ChoixTypeDlg(frame, info.typesProches);
		    		dlgct.setLocationRelativeTo(frame);
		        dlgct.setVisible(true);
		        if (!dlgct.retourOK()) return;    
		        int indType = dlgct.indiceTypeChoisi;
		        // Il faut regarder si dans le type choisi un seul objet est proche ou plusieurs
		        if (info.infoParType[indType].nombreVoisins == 1) {
		          elg = info.infoParType[indType].premierVoisin;
		        }
		        else { // Plusieurs objets du type choixi sont proches du curseur
		          PremierDernierDlg dlg = new PremierDernierDlg(frame);
		      		dlg.setLocationRelativeTo(frame);
		          dlg.setVisible(true);
		          if (!dlg.getRetourOK()) return;
		          // On regarde quelle a t la rponse de l'utilisateur
		          if (dlg.getPremierChoisi()) elg = info.infoParType[indType].premierVoisin;
		          else elg = info.infoParType[indType].dernierVoisin;        
		        }  
		      }
		      frame.outilActif.traiteObjetDesigne(elg, mouseEvent.getPoint());
	    	}
	    }
	  }
  	else { // Cas d'un appui sur le bouton droit. On finit alors la cration de l'objet
  		frame.outilActif.creeObjetApresClicDroit();
  	}
  }
  
  public final boolean objetsSecants(CElementGraphique elg1, CElementGraphique elg2) {
  	if ((elg1 == null) || (elg2 == null)) return false;
  	long nat1 = elg1.getNature();
  	long nat2 = elg2.getNature();
  	if (((nat1 & NatureObjet.NTouteDroiteSaufVecteur) != 0) 
  			& ((nat2 & NatureObjet.NTouteDroiteSaufVecteur) != 0)) {
  		Point2D.Double pointInt1 = new Point2D.Double();
  		CDroiteAncetre dte1 = (CDroiteAncetre) elg1;
  		CDroiteAncetre dte2 = (CDroiteAncetre) elg2;

  		byte positionRelative = Kernel.intersection(dte1.point_x, dte1.point_y, dte1.vecteur, dte2.point_x,
  				dte2.point_y, dte2.vecteur, pointInt1);
  		return (positionRelative == PositionDroites.Secantes);
  	}
  	else {
  		if (((nat1 & NatureObjet.NToutCercle) != 0) & ((nat2 & NatureObjet.NToutCercle) != 0)) {
  			// Coordonnes des points d'intersection
  			CCercle cerc1 = (CCercle) elg1;
  			CCercle cerc2 = (CCercle) elg2;
  			Point2D.Double pointInt1 = new Point2D.Double();
  			Point2D.Double pointInt2 = new Point2D.Double();
  			byte positionRelative = Kernel.intersectionCercleCercle(cerc1.centre_x, cerc1.centre_y, cerc1.rayon,
  					cerc2.centre_x, cerc2.centre_y, cerc2.rayon, pointInt1, pointInt2);
  			return (positionRelative == PositionCercleCercle.Secants) || (positionRelative == PositionCercleCercle.Tangents);
  		}
  		else {
  			CDroiteAncetre ptd;
  			CCercle ptc;
  			if ((nat1 & NatureObjet.NTouteDroiteSaufVecteur) != 0) {
  				ptd = (CDroiteAncetre) elg1;
  				ptc = (CCercle) elg2;
  			}
  			else {
  				ptd = (CDroiteAncetre) elg2;
  				ptc = (CCercle) elg1;
  			}
  			// Coordonnes des points d'intersection
  			Point2D.Double pointInt1 = new Point2D.Double();
  			Point2D.Double pointInt2 = new Point2D.Double();
  			byte positionRelative = Kernel.intersectionDroiteCercle(ptd.point_x, ptd.point_y, ptd.vecteur,
  					ptc.centre_x, ptc.centre_y, ptc.rayon, pointInt1, pointInt2);
  			return (positionRelative != PositionDroiteCercle.Vide);
  		} 
  	}
  }  
}
